#include<string>
#include <sys/time.h>
#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkm3d.h"

#include "libeuler/pomEuler.h"

#ifdef JIDO
#include "fingers/server/fingersPosterLibStruct.h"
#endif

#if defined(ATTENTIONAL)

/*----------------------------------------------------------------
 * Compute Attentional Data
 *
 * Description:
 *
 */

/*
   Returns:  EXEC END ETHER FAIL ZOMBIE */
/*
typedef struct ATTENTIONAL_INPUT_STR{
  ATTENTIONAL_HUMAN_STR human[UNINA_MAX_HUMAN];
  ATTENTIONAL_ROBOT_STR robot;
  ATTENTIONAL_OBJECT_STR object[UNINA_MAX_OBJ];
} ATTENTIONAL_INPUT_STR;

typedef enum ATTENTIONAL_ACTION_ENUM {
  ATTENTIONAL_GIVING,
  ATTENTIONAL_PICKING,
  ATTENTIONAL_PLACING,
  ATTENTIONAL_RECEIVING,
  ATTENTIONAL_INACTIVE
} ATTENTIONAL_ACTION_ENUM;

typedef struct ATTENTIONAL_ROBOT_STR{
  GEN_STRING128 name;
  UNINA_POS base_pos;
  UNINA_POS base_vel;
  UNINA_POS gripper_pos;
  UNINA_POS gripper_vel;
  ATTENTIONAL_ACTION_ENUM current_action ;
  int current_object;
  int current_partner;
} ATTENTIONAL_ROBOT_STR;


*/

const int MAX_OBJECT_ATTENTIONAL_OBJECTS = 20;

static unsigned int attentionalIter;
static ATTENTIONAL_INPUT_STR attentionalInputOld;
static double attentionalObjectLastTimeSeen[MAX_OBJECT_ATTENTIONAL_OBJECTS];

void sparkAttentionalPrintPos( const char* name, const UNINA_POS& pos )
{
  if( pos.available == FALSE )
    {
      printf("Attentional : %s unavailable\n");
    }
  else
    {
      p3d_vector3 pos3d;
      pos3d[0] = pos.x;
      pos3d[1] = pos.y;
      pos3d[2] = pos.z;

      double norm = p3d_vectNorm( pos3d );
      printf("Attentional : %s, (%f,%f,%f), norm(%f)\n",name,pos.x,pos.y,pos.z,norm);
    }
}

void sparkAttentionalPrintRobot()
{
  ATTENTIONAL_ROBOT_STR *DATA_ROB = &(SDI_F->attentional_str.robot);
  printf("------------------------------------------------------\n" );
  printf("Attentional : ROBOT, NAME : %s\n", DATA_ROB->name.name );

  sparkAttentionalPrintPos( "base_pos", DATA_ROB->base_pos );
  sparkAttentionalPrintPos( "base_vel", DATA_ROB->base_vel );
  sparkAttentionalPrintPos( "gripper_pos", DATA_ROB->gripper_pos );
  sparkAttentionalPrintPos( "gripper_vel", DATA_ROB->gripper_vel );
  printf("Attentional : holding : %d\n", DATA_ROB->holding );
}

void sparkAttentionalPrintHuman(int id)
{
  ATTENTIONAL_HUMAN_STR *DATA_HUM = &(SDI_F->attentional_str.human[id]);
  printf("------------------------------------------------------\n" );
  printf("Attentional : Human(%d), NAME : %s IS IN THE SCENE\n", id, DATA_HUM->name.name );

  sparkAttentionalPrintPos( "base_pos" , DATA_HUM->base_pos );
  sparkAttentionalPrintPos( "base_vel" , DATA_HUM->base_vel );
  printf("Attentional : cost_dist_to_robot : %f\n", DATA_HUM->cost_dist_to_robot );

  sparkAttentionalPrintPos( "hand_left_pos", DATA_HUM->hand_left_pos );
  sparkAttentionalPrintPos( "hand_left_vel", DATA_HUM->hand_left_vel );
  sparkAttentionalPrintPos( "hand_right_pos", DATA_HUM->hand_right_pos );
  sparkAttentionalPrintPos( "hand_right_vel", DATA_HUM->hand_right_vel );
  printf("Attentional : is_looking_to_robot : %f\n", DATA_HUM->is_looking_to_robot );
}

void sparkAttentionalPrintObject(int id)
{
  ATTENTIONAL_OBJECT_STR *DATA_OBJ = &(SDI_F->attentional_str.object[id]);

  printf("------------------------------------------------------\n" );
  if( DATA_OBJ->is_in_the_scene )
    printf("Attentional : Object(%d), NAME : %s IS IN THE SCENE\n", id, DATA_OBJ->name.name );
  else
    printf("Attentional : Object(%d), NAME : %s NOT IN THE SCENE\n", id, DATA_OBJ->name.name );

  sparkAttentionalPrintPos( "base_pos", DATA_OBJ->pos );
  sparkAttentionalPrintPos( "base_vel" , DATA_OBJ->vel );
}

void sparkAttentionPrintData()
{

  sparkAttentionalPrintRobot();

    for(int i=0; i<gAgents->humans_no;i++)
    {
      HRI_AGENT* human = gAgents->humans[i];

      if( SDI_F->attentional_str.human[i].is_in_the_scene )
        {
          sparkAttentionalPrintHuman(i);
        }
      else
        {
          printf("------------------------------------------------------\n" );
          printf("Attentional : Human(%d), NAME : %s IS NOT INT THE SCENE\n", i, gAgents->humans[i]->robotPt->name );
        }
    }

    sparkAttentionalPrintObject(0);
    sparkAttentionalPrintObject(1);
}

int sparkAttentionalFillRobotData()
{
  p3d_matrix4 T_robot_origin,T_tmp;
  p3d_jnt* jntPt=NULL;
  p3d_vector3 pos3d;

  ATTENTIONAL_ROBOT_STR *DATA_ROB = &(SDI_F->attentional_str.robot);

  // Get the human name from the ID
  strcpy( DATA_ROB->name.name , gRobot->robotPt->name );

  // Get the previous attentional data
  ATTENTIONAL_ROBOT_STR *OLD_DATA_ROB = &(attentionalInputOld.robot);

  // Get the delta time to previous
  double dt_sec  = SDI_F->attentional_str.tacq_sec - attentionalInputOld.tacq_sec;
  double dt_usec = SDI_F->attentional_str.tacq_usec - attentionalInputOld.tacq_usec;
  double dt = dt_sec+dt_usec/1000000.0;

  // Get the base position and velocity
  p3d_jnt_get_cur_vect_point( gRobot->robotPt->joints[1], pos3d );
  DATA_ROB->base_pos.available = TRUE;
  DATA_ROB->base_pos.x = pos3d[0];
  DATA_ROB->base_pos.y = pos3d[1];
  DATA_ROB->base_pos.z = pos3d[2];

  DATA_ROB->base_vel.available = TRUE;
  DATA_ROB->base_vel.x = ( pos3d[0] - OLD_DATA_ROB->base_pos.x ) / dt;
  DATA_ROB->base_vel.y = ( pos3d[1] - OLD_DATA_ROB->base_pos.y ) / dt;
  DATA_ROB->base_vel.z = ( pos3d[2] - OLD_DATA_ROB->base_pos.x ) / dt;

  // Get the gripper position and velocity
  jntPt = p3d_get_robot_jnt_by_name(gRobot->robotPt, (char*)"RightWrist");
  p3d_jnt_get_cur_vect_point( jntPt, pos3d );
  if(jntPt) {
    DATA_ROB->gripper_pos.available = TRUE;
    DATA_ROB->gripper_pos.x = pos3d[0];
    DATA_ROB->gripper_pos.y = pos3d[1];
    DATA_ROB->gripper_pos.z = pos3d[2];
  } else {
    DATA_ROB->gripper_pos.available = FALSE;
  }
  DATA_ROB->gripper_vel.available = TRUE;
  DATA_ROB->gripper_vel.x = ( pos3d[0] - OLD_DATA_ROB->gripper_pos.x ) / dt;
  DATA_ROB->gripper_vel.y = ( pos3d[1] - OLD_DATA_ROB->gripper_pos.y ) / dt;
  DATA_ROB->gripper_vel.z = ( pos3d[2] - OLD_DATA_ROB->gripper_pos.z ) / dt;

  // Get the current task
  DATA_ROB->current_task.action  = SDI_F->attentional_task.action;
  DATA_ROB->current_task.object  = SDI_F->attentional_task.object;
  DATA_ROB->current_task.partner = SDI_F->attentional_task.partner;

  //DATA_ROB->holding = gRobot->robotPt->    ;
  return 0;
}


int sparkAttentionalFillHumanData(ATTENTIONAL_HUMAN_STR *DATA_HUM, int HUMAN_ID)
{
  p3d_matrix4 T_robot_origin,T_tmp;
  p3d_jnt* jntPt=NULL;

  p3d_vector3 robot_pos;
  p3d_vector3 human_pos;
  p3d_vector3 pos3d;
  p3d_vector3 Origin = { 0.0 , 0.0 , 0.0 };

  // Get the human model;
  HRI_AGENT* humanAgent = gAgents->humans[HUMAN_ID];

  // Checks if the human is present in the scene
  DATA_HUM->is_in_the_scene =  humanAgent->is_present;

  // Get the human name from the ID
  strcpy( DATA_HUM->name.name , humanAgent->robotPt->name );

  // Check that the human is in the scene
  if( humanAgent->is_present == true )
    {
      // Get the human position in world frame
      // Check that the human is at a certain distance
      // from the Origin
      p3d_jnt_get_cur_vect_point( humanAgent->robotPt->joints[1], human_pos );

      if(  p3d_vectDistance( human_pos , Origin ) < 0.5 )
        {
          DATA_HUM->is_in_the_scene = FALSE;
        }
      else {
        DATA_HUM->is_in_the_scene = TRUE;
      }
    }

  if( DATA_HUM->is_in_the_scene == FALSE )
    {
      return 1;
    }

  // Get the previous attentional data
  ATTENTIONAL_HUMAN_STR *OLD_DATA_HUM = &(attentionalInputOld.human[HUMAN_ID]);

  // Get the delta time to previous
  double dt_sec  = SDI_F->attentional_str.tacq_sec - attentionalInputOld.tacq_sec;
  double dt_usec = SDI_F->attentional_str.tacq_usec - attentionalInputOld.tacq_usec;
  double dt = dt_sec+dt_usec/1000000.0;

  // Store the human base position and velocity
  DATA_HUM->base_pos.available = TRUE;
  DATA_HUM->base_pos.x = human_pos[0];
  DATA_HUM->base_pos.y = human_pos[1];
  DATA_HUM->base_pos.z = human_pos[2];

  DATA_HUM->base_vel.available = TRUE;
  DATA_HUM->base_vel.x = ( human_pos[0] - OLD_DATA_HUM->base_pos.x ) / dt;
  DATA_HUM->base_vel.y = ( human_pos[1] - OLD_DATA_HUM->base_pos.y ) / dt;
  DATA_HUM->base_vel.z = ( human_pos[2] - OLD_DATA_HUM->base_pos.z ) / dt;

  // Compute the miniman distance between
  // this human and the robot
  DATA_HUM->cost_dist_to_robot = hri_distance_cost( gRobot,
                                                    gAgents->humans[HUMAN_ID],
                                                    DATA_HUM->minimal_dist_to_robot);


  // Store the human right hand position and velocity
  jntPt = p3d_get_robot_jnt_by_name( humanAgent->robotPt, "rWristX");
  p3d_jnt_get_cur_vect_point( jntPt, pos3d );
  DATA_HUM->hand_right_pos.available = FALSE;
  if(jntPt) {
    DATA_HUM->hand_right_pos.available = TRUE;
    DATA_HUM->hand_right_pos.x = pos3d[0];
    DATA_HUM->hand_right_pos.y = pos3d[1];
    DATA_HUM->hand_right_pos.z = pos3d[2];
  }
  DATA_HUM->hand_right_vel.available = TRUE;
  DATA_HUM->hand_right_vel.x = ( pos3d[0] - OLD_DATA_HUM->hand_right_pos.x ) / dt;
  DATA_HUM->hand_right_vel.y = ( pos3d[1] - OLD_DATA_HUM->hand_right_pos.y ) / dt;
  DATA_HUM->hand_right_vel.z = ( pos3d[2] - OLD_DATA_HUM->hand_right_pos.z ) / dt;

  // Store the human left hand position and velocity
  jntPt = p3d_get_robot_jnt_by_name( humanAgent->robotPt, "lWristX");
  p3d_jnt_get_cur_vect_point( jntPt, pos3d );
  DATA_HUM->hand_left_pos.available = FALSE;
  if(jntPt) {
    DATA_HUM->hand_left_pos.available = TRUE;
    DATA_HUM->hand_left_pos.x = pos3d[0];
    DATA_HUM->hand_left_pos.y = pos3d[1];
    DATA_HUM->hand_left_pos.z = pos3d[2];
  }
  DATA_HUM->hand_left_vel.available = TRUE;
  DATA_HUM->hand_left_vel.x = ( pos3d[0] - OLD_DATA_HUM->hand_left_pos.x ) / dt;
  DATA_HUM->hand_left_vel.y = ( pos3d[1] - OLD_DATA_HUM->hand_left_pos.y ) / dt;
  DATA_HUM->hand_left_vel.z = ( pos3d[2] - OLD_DATA_HUM->hand_left_pos.z ) / dt;

  // Get the position of the endeffector of the robot
  jntPt = p3d_get_robot_jnt_by_name(gRobot->robotPt, (char*)"RightWrist");
  p3d_jnt_get_cur_vect_point( jntPt, pos3d );
  robot_pos[0] = pos3d[0];
  robot_pos[1] = pos3d[1];
  robot_pos[2] = pos3d[2];

  // Get the robot endeffector visibility from the human
  double visiCost = hri_simple_is_point_visible_by_robot( robot_pos, humanAgent->robotPt );
  if(visiCost > 0.3) {
    DATA_HUM->is_looking_to_robot = 0.0;
  } else {
    DATA_HUM->is_looking_to_robot = 1.0;
  }

  return 0;
}

int sparkAttentionalFillObjectData(char* object_name, ATTENTIONAL_OBJECT_STR *DATA_OBJ,  int OBJ_ID)
{
  VimanObjectPublicArray vimanPoster;
  p3d_rob* Object = NULL;
  bool objectFound;
  p3d_jnt* jntPt=NULL;
  p3d_vector3 pos3d;

  // Get Object Name
  strcpy(DATA_OBJ->name.name , object_name);

  DATA_OBJ->pos.available = FALSE;
  DATA_OBJ->vel.available = FALSE;

  if(SDI_F->state.vimanPosterAv) {
    if(posterRead(gVimanPosterID,0, &vimanPoster, sizeof(vimanPoster)) != sizeof(vimanPoster)) {
      return 1;
    }
  }

  objectFound = FALSE;
  int i = 0;

  for(i=0; i<vimanPoster.nbObjects; i++) {
    if (!strcmp(vimanPoster.objects[i].name, object_name )) {
      objectFound = TRUE;
      break;
    }
  }

  Object = p3d_get_robot_by_name( object_name );

  if( !( objectFound && Object ) )
    {
      DATA_OBJ->is_in_the_scene = 0 ;
      return 0;
    }
  else
    {
      DATA_OBJ->pos.available = TRUE;
      p3d_jnt_get_cur_vect_point( Object->joints[1], pos3d );
      DATA_OBJ->pos.x = pos3d[0];
      DATA_OBJ->pos.y = pos3d[1];
      DATA_OBJ->pos.z = pos3d[2];

      // Get the previous attentional data
      ATTENTIONAL_OBJECT_STR *OLD_DATA_OBJ = &(attentionalInputOld.object[OBJ_ID]);

      // Get the delta time to previous
      double dt_sec  = SDI_F->attentional_str.tacq_sec - attentionalInputOld.tacq_sec;
      double dt_usec = SDI_F->attentional_str.tacq_usec - attentionalInputOld.tacq_usec;
      double dt = dt_sec+dt_usec/1000000.0;

      DATA_OBJ->vel.available = TRUE;
      DATA_OBJ->vel.x = ( pos3d[0] - OLD_DATA_OBJ->pos.x ) / dt;
      DATA_OBJ->vel.y = ( pos3d[1] - OLD_DATA_OBJ->pos.y ) / dt;
      DATA_OBJ->vel.z = ( pos3d[2] - OLD_DATA_OBJ->pos.z ) / dt;

      double time = SDI_F->attentional_str.tacq_sec+SDI_F->attentional_str.tacq_usec/1000000.0;

      if( vimanPoster.objects[i].found_Stereo == 1 )
        {
          DATA_OBJ->is_in_the_scene = 1 ;
          attentionalObjectLastTimeSeen[OBJ_ID] = time;
        }
      else
        {
          // When the object has not been seen
          // for more than 2 seconds it is marked as not present in the scene
          if( time - attentionalObjectLastTimeSeen[OBJ_ID] > 4.0 )
            {
              //printf("Set object %s to origin\n",Object->name);
              DATA_OBJ->is_in_the_scene = 0 ;
              configPt q = p3d_get_robot_config( Object );
              q[6] = 0;
              q[7] = 0;
              q[8] = 0;
              q[9] = 0;
              q[10] = 0;
              q[11] = 0;
              p3d_set_and_update_this_robot_conf( Object, q );
              place_robot_conf_to_environment_poster( Object, q );
            }
          else
            {
              DATA_OBJ->is_in_the_scene = 1;
            }
        }
    }

  return 0;
}


ACTIVITY_EVENT
sparkComputeAttentionalDataMain(int *report)
{
  timeval tv;
  gettimeofday(&tv, NULL);
  SDI_F->attentional_str.tacq_sec = tv.tv_sec;
  SDI_F->attentional_str.tacq_usec = tv.tv_usec;

  /*****************************************/
  /*  Get data of the robot                */
  /*****************************************/
  sparkAttentionalFillRobotData();

  /*****************************************/
  /*  Get data of the humans               */
  /*****************************************/

  ATTENTIONAL_HUMAN_STR *DATA_HUM1 = &(SDI_F->attentional_str.human[0]);
  sparkAttentionalFillHumanData(DATA_HUM1, 0);

  ATTENTIONAL_HUMAN_STR *DATA_HUM2 = &(SDI_F->attentional_str.human[1]);
  sparkAttentionalFillHumanData(DATA_HUM2, 1);

  /*****************************************/
  /*  Get data of the objects              */
  /*****************************************/
  ATTENTIONAL_OBJECT_STR *DATA_OBJ1 = &(SDI_F->attentional_str.object[0]);
  sparkAttentionalFillObjectData("GREY_TAPE", DATA_OBJ1,  0);

  ATTENTIONAL_OBJECT_STR *DATA_OBJ2 = &(SDI_F->attentional_str.object[1]);
  sparkAttentionalFillObjectData("LOTR_TAPE", DATA_OBJ2,  1);

  /************************/
  /*  Write the poster    */
  /************************/
  sparkAttentionalDataattentional_strPosterWrite  ( SPARK_ATTENTIONALDATA_POSTER_ID ,
                                                    &SDI_F->attentional_str);

  if( SDI_F->printAttentionalData == GEN_TRUE )
    sparkAttentionPrintData();

  attentionalInputOld = SDI_F->attentional_str;
  attentionalIter++;

  publish_environment_poster();

  return EXEC;
}

#endif // ATTENTIONAL
