/**
 ** sparkMainTaskCodels.cc
 **
 ** Codels called by execution task sparkMainTask
 **
 ** Author: Emrah Akin Sisbot
 ** Date: Aug 2010
 **
 **/

#include <portLib.h>



#include "server/sparkHeader.h"
#include "sparkm3d.h"

#ifdef BETA_INTERFACE
#include <pthread.h>
#include <sys/time.h>
#include <taskLib.h>
#endif

#if USE_GLUT
#include <libmove3d/graphic/proto/g3d_glut.hpp>
GlutWindowDisplay* glutWin=NULL;
#endif

/* ---- Module internal variables with mostly Move3d types ----- */

p3d_env * gEnv = NULL;
HRI_AGENT  * gRobot = NULL;
HRI_AGENTS * gAgents = NULL;
HRI_ENTITIES * gEntities = NULL;
HRI_ACTION_MONITORING_SPHERES * gSpheres = NULL; // Spheres used for action monitoring
bool initSpheresIsDone= false; // Are spheres used for action monitoring initialized?
/* ---- Module static functions ----- */

static int fillEnvPoster();
static int fillBoundingBoxesPoster();
static int initialize_fact_operations();
void initializePositionInfo();

static bool initIsDone= false;

/* ------------------------------------------- */
#ifdef BETA_INTERFACE
void *threadEventLoopFct(void *) {

  printf("Qt main Thread\n");
  initialize_interface();

  return 0;
}
#endif
/* ------------------------------------------- */

/*------------------------------------------------------------------------
 * LoadP3d
 *
 * Description:
 *
 * Reports:      OK
 *              S_spark_FILE_NOT_FOUND
 *              S_spark_ERROR_UNKNOWN
 *              S_spark_SIT_ASSESS_NOT_INITIALIZED
 */

/* sparkLoadP3dMain  -  codel EXEC of LoadP3d
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkLoadP3dMain(SPARK_P3D *p3d, int *report)
{
  int col_mode = p3d_col_mode_pqp;
  int FILTER_TO_BE_SET_ACTIVE = FALSE ;
  char *argv[2];
  int argc;
  int i;

  printf("\n****************** LAUNCHING SPARK *********************\n");

  printf("%s ......loading\n", p3d->p3dFileName.name);

  argv[0] = (char*) "SPARK";
  argv[1] = p3d->p3dFileName.name;
  argc = 2;

  p3d_init_random();
  set_collision_by_object(FALSE);

  p3d_filter_switch_filter_mechanism(FILTER_TO_BE_SET_ACTIVE);

  if(SDI_F->p3d.enableGraphic) {
#ifdef USE_GLUT
    //Initialize GLUT windows, only if not already done in a previous call to LoadP3D
    if (glutWin == NULL) glutWin = new GlutWindowDisplay(argc,argv);
#else

#endif
  }

  while(!p3d_get_desc_number(P3D_ENV)) {
    p3d_col_set_mode(p3d_col_mode_none);
    p3d_BB_set_mode_close();

    if ( p3d_read_desc((char *) p3d->p3dFileName.name) == FALSE){
      *report =  S_spark_FILE_NOT_FOUND;
      printf("Bad file type or file does not exist\n");
      return ETHER;
    }

    if(!p3d_get_desc_number(P3D_ENV)) {
      printf("\n error while loading p3d.\n");
      *report =  S_spark_ERROR_UNKNOWN;
      return ETHER;
    }
  }

  p3d_col_set_mode(col_mode);
  p3d_col_start(col_mode);

  if(SDI_F->p3d.enableGraphic) {
#ifdef USE_GLUT
    glutWin->initDisplay();
#else
#ifdef BETA_INTERFACE
    printf("Intializes interface!!!!\n");

    if (taskSpawn("loopEvent",
                  1 /* priorite */,
                  VX_FP_TASK,
                  8000000 /*size*/,
                  (FUNCPTR)threadEventLoopFct) == ERROR) {
      printf("cannot spawn loop event thread\n");
      return ETHER;
    }
#endif
#endif
    initialize_interface();
  }

  /* --------- Internal data and Poster Initialization -------- */

  SDI_F->state.posPosterAv         = FALSE;
  SDI_F->state.armConfPosterAv     = FALSE;
  SDI_F->state.handPosterAv        = FALSE;
  SDI_F->state.ptPosterAv          = FALSE;
  SDI_F->state.robotConfPosterAv   = FALSE;
  SDI_F->state.vimanPosterAv       = FALSE;
  SDI_F->state.objectsFromKinectPosterAv = FALSE;
  SDI_F->state.sparkObjPosPosterAv = FALSE;
  SDI_F->state.useObjInsideSparkEnabled = FALSE;
  SDI_F->state.hrp2ConfPosterAv    = FALSE;
  SDI_F->state.mocapPosterAv       = FALSE;
  SDI_F->state.mocapHandPosterAv   = FALSE;
  SDI_F->state.humanConfPosterAv   = FALSE;
  SDI_F->state.bodyPartsPosterAv   = FALSE;
  SDI_F->state.oroConnectionAv     = FALSE;
  SDI_F->state.oroPrintEnabled     = FALSE;
  SDI_F->state.vimanPrintEnabled   = FALSE;
  SDI_F->state.inhibitLookDirectionOroFactsSending = FALSE;
  SDI_F->state.reexportReachabilityFactsToOro = FALSE;
  SDI_F->state.fillPosePerceptionInfoPoster = FALSE;
  SDI_F->state.fillPosePerceptionInfoPosterStartReady = FALSE;

  SDI_F->interfaceParams.width = 800;
  SDI_F->interfaceParams.height = 600;
  SDI_F->interfaceParams.saveInterface = GEN_FALSE;
  SDI_F->interfaceParams.floor         = GEN_TRUE;
  SDI_F->interfaceParams.walls         = GEN_TRUE;
  SDI_F->interfaceParams.shadows       = GEN_FALSE;
  SDI_F->interfaceParams.tiles         = GEN_FALSE;
  SDI_F->interfaceParams.antialiasing  = GEN_FALSE;

  //////////////////////
  //request time tracking
  ///////////////////////

  SDI_F->requestTimeInfo.numRequestTracked = 0;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeEnabled = FALSE;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintPeriod = 5;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintIter = 0;

  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadObjectsIndex = -1;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadRobotIndex = -1;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadHumansIndex = -1;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeComputeFactsIndex = -1;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeTestSpheresIndex = -1;
  SDI_F->requestTimeInfo.printLoopRequestExecutionTimeDistanceInfoIndex = -1;
  for(int i = 0; i < SPARK_REQUEST_TIME_MAX ; i++){
    SDI_F->requestTimeInfo.maxTime[i]=0;
    SDI_F->requestTimeInfo.timeSinceBeginning[i]=0;
  }

  ////////////////
  /// Visu Info Management in spark. To Comment after the demo!
  //////////////////
  SDI_F->visuInfoManagement.showReachability = GEN_TRUE;
  SDI_F->visuInfoManagement.showDivergentBelief = GEN_FALSE;
  SDI_F->visuInfoManagement.showTablePoints = GEN_FALSE;

  SDI_F->visuInfoManagement.radius = 0.04;
  SDI_F->visuInfoManagement.opacity = 1.0;
  SDI_F->visuInfoManagement.verticalOffSet.x = 0;
  SDI_F->visuInfoManagement.verticalOffSet.y = 0;
  SDI_F->visuInfoManagement.verticalOffSet.z = 0.3;
  SDI_F->visuInfoManagement.distBetweenSphereCenters = 0.05;
  SDI_F->visuInfoManagement.distMaxBeforeUpdate = 0.1;
  for(int i = 0; i < SPARK_VISU_AGENT_MAX ; i++){
    for(int j = 0; j < SPARK_VISU_REACHABILITY_BY_AGENT_MAX; j++){
      SDI_F->visuInfoManagement.showReachabilityEntityVisuMapping[i][j] = -1;
    }
    for(int j = 0; j < SPARK_VISU_DIVERGENT_BELIEF_BY_AGENT_MAX; j++){
      SDI_F->visuInfoManagement.showDivergentBeliefEntityVisuMapping[i][j] = -1;
    }
    SDI_F->visuInfoManagement.horizontalOffSet[i].x = -(SPARK_VISU_AGENT_MAX-1)*SDI_F->visuInfoManagement.distBetweenSphereCenters/2 +i*SDI_F->visuInfoManagement.distBetweenSphereCenters;
    SDI_F->visuInfoManagement.horizontalOffSet[i].y = 0;
    SDI_F->visuInfoManagement.horizontalOffSet[i].z = 0;
    SDI_F->visuInfoManagement.r[i]=0.0;
    SDI_F->visuInfoManagement.g[i]=0.0;
    SDI_F->visuInfoManagement.b[i]=0.0;
  }
  if(SPARK_VISU_AGENT_MAX>3){
    SDI_F->visuInfoManagement.r[0] = 1.0;
    SDI_F->visuInfoManagement.g[1] = 1.0;
    SDI_F->visuInfoManagement.b[2] = 1.0;
  }

  //Added by mw to simu input in Spark from external sources
  SDI_F->addedObjectSimu.isnew = 0;

  strcpy(SDI_F->oroServerAddress.address.name, "localhost");
  strcpy(SDI_F->oroServerAddress.port.name, "6969");

  /* -------------------------------------------------------------------- */

  gEnv = (p3d_env *) p3d_get_desc_curid(P3D_ENV);

  /* Initialize Agents structure */
  gAgents = hri_create_agents();
  hri_assign_global_agents(gAgents);
#ifdef JIDO
  gRobot = hri_assign_source_agent((char*)"JIDOKUKA_ROBOT", gAgents);
#elif HRP2
  gRobot = hri_assign_source_agent((char*)"HRP2_ROBOT", gAgents);
#elif PR2
  gRobot = hri_assign_source_agent((char*)"PR2_ROBOT", gAgents);
#else
  gRobot = hri_assign_source_agent(gAgents->robots[0]->robotPt->name, gAgents);
#endif
  /* End Initialize Agents */

  if (gRobot == NULL || gRobot->robotPt == NULL) {
    printf("Cannot chose the current robot\n");
    *report = S_spark_ERROR_UNKNOWN;
    return FAIL;
  }

  /* Create and initialize entitities */
  gEntities = hri_create_entities();
  hri_link_agents_with_entities(gEntities, gAgents);
  hri_initialize_all_agents_knowledge(gEntities, gAgents);
  /* End initialize entities */
  hri_display_entities(gEntities);

  p3d_init_iksol(gRobot->robotPt->cntrt_manager);

  /* Set the robots to initial Pos if defined in p3d file */
  /* Deactivate cc constraints to avoid bug with not moving bounding boxes */
  for(i = 0; i<gEnv->nr; i++){
    if(strcasestr(gEnv->robot[i]->name, "ROBOT"))
      deactivateCcCntrts(gEnv->robot[i],-1);
    if(!p3d_isNullConfig(gEnv->robot[i], gEnv->robot[i]->ROBOT_POS)){
      p3d_set_and_update_this_robot_conf(gEnv->robot[i], gEnv->robot[i]->ROBOT_POS);
    }
  }

  if(constructP3dToRobotNumIndexTable() == FALSE) {
    *report = S_spark_NOT_INITIALIZED;
    initIsDone=false;
    return ETHER;
  }

  fillEnvPoster();
  fillBoundingBoxesPoster();

  initialize_fact_operations();

  initializePositionInfo();

  if(p3d->enableMight) {
    //TODO: Check if old mightability code works
    //Create_and_init_Mightability_Maps();
    SDI_F->state.mightabilitiesEnabled = TRUE;
  }
  else {
    SDI_F->state.mightabilitiesEnabled = FALSE;
  }

#if defined (HRI_COSTSPACE)
  if( getenv("HOME_MOVE3D") != NULL )
    {
      Graphic::initDrawFunctions();
      XYZ_ENV->active_robot = gRobot->robotPt;
      global_ActiveRobotName = gRobot->robotPt->name;
      global_Project = new Project(new Scene(XYZ_ENV));

      //HRICS_init();
      //ENV.setBool(Env::drawGrid,true);
      //ENV.setBool(Env::isCostSpace,false);
    }
  else
    {
      printf("HRICS : HOME_MOVE3D is not defined -> not possible to launch!!!\n");
    }
#endif

  printf ("\n****************** SPARK LAUNCHED *********************\n");
  SDI_F->safetyCostMode = GEN_FALSE;
  *report = OK;
  initIsDone=true;
  return ETHER;
}

/*------------------------------------------------------------------------
 * LoadSce
 *
 * Description:
 *
 * Reports:      OK
 *              S_spark_FILE_NOT_FOUND
 *              S_spark_NOT_INITIALIZED
 */

/* sparkLoadSceMain  -  codel EXEC of LoadSce
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkLoadSceMain(SPARK_SCE *sce, int *report)
{
  int i;

  if(gEnv == NULL){
    printf("SPARK is not properly initialized.\n");
    *report = S_spark_NOT_INITIALIZED;
    return ETHER;
  }

  p3d_read_scenario(sce->name.name);

  for(i=0; i<gEnv->nr; i++){
    p3d_set_and_update_this_robot_conf(gEnv->robot[i], gEnv->robot[i]->ROBOT_POS);
  }
  fillEnvPoster();
  return ETHER;
}


/* Functions putting the name and configuration of all Move3D robots into a poster */
static int fillEnvPoster()
{
  int i;

  if(gEnv == NULL){
    printf("SPARK is not properly initialized.\n");
    return FALSE;
  }

  if(gEnv->nr > SPARK_MAX_THINGS_NB) {
    printf("Poster is too small for this environment. Maximum robot no:%d\n", SPARK_MAX_THINGS_NB);
    return FALSE;
  }

  strcpy(SDI_F->envCur.envName.name, gEnv->name);

  //SDI_F->envCur.robotNb = gEnv->nr;

  for(i=0; i<gEnv->nr; i++)
  {

    if(!place_robot_conf_to_environment_poster(gEnv->robot[i], gEnv->robot[i]->ROBOT_POS))
      {
       printf("Can not place robot in the environement poster\n");
       return FALSE;
      }
  }

  publish_environment_poster();

  return TRUE;
}

/* Function putting the name and the bounding boxes of all the robots in a poster */
/* Human is divided to parts and written that way */
static int fillBoundingBoxesPoster()
{
  int i, j, idx;

  if(gEnv == NULL){
    printf("SPARK is not properly initialized.\n");
    return FALSE;
  }

  strcpy(SDI_F->BoundingBoxes.envName.name, gEnv->name);

  idx = 0;

  for(i=0; i<gEnv->nr && i<SPARK_MAX_THINGS_NB ; i++){
    if(strcasestr(gEnv->robot[i]->name, "HUMAN")) {
      /* For human we export the bboxes of head/hands/torso */
      for(j=0; j<gEnv->robot[i]->no; j++) {
        if(!strcasestr(gEnv->robot[i]->o[j]->name, "GHOST") &&
           (strcasestr(gEnv->robot[i]->o[j]->name, "HEAD") ||
            strcasestr(gEnv->robot[i]->o[j]->name, "HAND") ||
            strcasestr(gEnv->robot[i]->o[j]->name, "TORSO")) ) {
          strcpy(SDI_F->BoundingBoxes.bbox[idx].name.name, gEnv->robot[i]->o[j]->name);
          SDI_F->BoundingBoxes.bbox[idx].x = gEnv->robot[i]->o[j]->box.x2 - gEnv->robot[i]->o[j]->box.x1;
          SDI_F->BoundingBoxes.bbox[idx].y = gEnv->robot[i]->o[j]->box.y2 - gEnv->robot[i]->o[j]->box.y1;
          SDI_F->BoundingBoxes.bbox[idx].z = gEnv->robot[i]->o[j]->box.z2 - gEnv->robot[i]->o[j]->box.z1;
          idx++;
        }
      }
    }
    else {
      strcpy(SDI_F->BoundingBoxes.bbox[idx].name.name, gEnv->robot[i]->name);
      SDI_F->BoundingBoxes.bbox[idx].x = gEnv->robot[i]->BB.xmax - gEnv->robot[i]->BB.xmin;
      SDI_F->BoundingBoxes.bbox[idx].y = gEnv->robot[i]->BB.ymax - gEnv->robot[i]->BB.ymin;
      SDI_F->BoundingBoxes.bbox[idx].z = gEnv->robot[i]->BB.zmax - gEnv->robot[i]->BB.zmin;
      idx++;
    }
  }
  SDI_F->BoundingBoxes.robotNb = idx;

  if(i==SPARK_MAX_THINGS_NB) {
    printf("Poster is too small for this environment. Maximum robot no:%d\n", SPARK_MAX_THINGS_NB);
    return FALSE;
  }

  sparkBoundingBoxesBoundingBoxesPosterWrite(SPARK_BOUNDINGBOXES_POSTER_ID, &SDI_F->BoundingBoxes);

  return TRUE;
}

/* Initialize Position Update Management Poster */
void initializePositionInfo(void)
{
  int i;
  printf("Initialize objectsPositionUpdateManagementState");
  SDI_F->positionsStatus.length = 0;
  SDI_F->positionsStatus.time = clock();
  for(i=0; i<SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX; i++) {
   strcpy(SDI_F->positionsStatus.objects[i].objectName.name, "");
   SDI_F->positionsStatus.objects[i].entityIndex = -1;
   SDI_F->positionsStatus.objects[i].lastPrecisePosition.x = 0;
   SDI_F->positionsStatus.objects[i].lastPrecisePosition.y = 0;
   SDI_F->positionsStatus.objects[i].lastPrecisePosition.z = 0;
   SDI_F->positionsStatus.objects[i].lastTimeSeenPrecisely = 0;

   SDI_F->positionsStatus.objects[i].lastRoughPosition.x = 0;
   SDI_F->positionsStatus.objects[i].lastRoughPosition.y = 0;
   SDI_F->positionsStatus.objects[i].lastRoughPosition.z = 0;
   SDI_F->positionsStatus.objects[i].lastTimeSeenRoughly = 0;

   SDI_F->positionsStatus.objects[i].distanceRoughToPrecise =0;
   SDI_F->positionsStatus.objects[i].detected = FALSE;
   SDI_F->positionsStatus.objects[i].detectedOnce = FALSE;

   publish_position_update_management_poster();
  }
}

/* Initialize situation assesment variables and values */
static int initialize_fact_operations(void)
{
  SDI_F->factOperations.curLineIndex = -1;
  SDI_F->factOperations.curFullTableIter = 0;
  SDI_F->factOperations.numLineMax = SPARK_FACT_OPERATION_MAX;
  SDI_F->factOperations.maxFactOperationsUpdateNum = 0;

  // SDI_F->factOperations.curLineIndex = 0;
  // SDI_F->factOperations.factOperation[0].type = SPARK_ADD_FACT;
  // GEN_STRING64 agentModel = { "HERAKLES_HUMAN1" };
  // GEN_STRING64 subject = { "GREY_TAPE" };
  // GEN_STRING64 predicate = { "isOn" };
  // GEN_STRING64 value = { "TABLE_4" };

  // strcpy(SDI_F->factOperations.factOperation[0].agentModel.name,agentModel.name);
  // strcpy(SDI_F->factOperations.factOperation[0].subject.name,subject.name);
  // strcpy(SDI_F->factOperations.factOperation[0].predicate.name,predicate.name);
  // strcpy(SDI_F->factOperations.factOperation[0].value.name,value.name);
  sparkFactOperationsfactOperationsPosterWrite(SPARK_FACTOPERATIONS_POSTER_ID, &SDI_F->factOperations);
  return TRUE;
}


/* sparkSaveModuleStateMain  -  codel EXEC of SaveModuleState
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
sparkSaveModuleStateMain(SPARK_SAVE_SCENARIO* saveSce, int *report)
{
  char fullPath[200];

  strcpy(fullPath, saveSce->savePath.name);
  strcat(fullPath, "/");
  strcat(fullPath, saveSce->fileName.name);

  if(!p3d_save_scenario(fullPath)) {
    *report = S_spark_CANNOT_SAVE_STATE;
    return ETHER;
  }
  else {
    *report = OK;
    return ETHER;
  }
}

/*------------------------------------------------------------
 *     Execution Tasks for a GLUT display
 *------------------------------------------------------------*/
#ifdef USE_GLUT
STATUS sparkMainTaskStart(int *report)
{

  return OK;
}

STATUS
sparkMainTaskEnd()
{
  return OK;
}

STATUS
sparkMainTaskPerm(int *report)
{
  if(initIsDone == false) {
    return OK;
  }
  if(SDI_F->p3d.enableGraphic==GEN_TRUE) {
    interface_MainLoopEvent();
  }
  return OK;
}
#endif




