#if defined(JIDO) || defined(HRP2) || defined(PR2)
#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkMocap.h"

double YAW_CALIBRATION=0;
double PITCH_CALIBRATION=0;
double CURRENT_YAW=0;
double CURRENT_PITCH=0;

void mocap_translate_markers_pos(MOCAP_OBJECTS *mocapObjects, double x, double y)
{
  int i,j;

  for(i = 0; i<mocapObjects->nbodies; i++) {
    for(j = 0; j<mocapObjects->objects[i].nmarkers; j++) {
      mocapObjects->objects[i].x[j] += x;
      mocapObjects->objects[i].y[j] += y;
    }
  }
}

int mocap_compute_neck_position_from_helmet(MOCAP_MARKERS_OBJECT *body, double *x, double *y, double *z, double *yaw, double *pitch)
{
  point_coords forehead_pos;
  double dx, dy, dz, dl;

  if(body == NULL)
    return FALSE;

  forehead_pos.x = (body->x[0] + body->x[1])/2; //0 is Cap front right and 1 is cap front left markers
  forehead_pos.y = (body->y[0] + body->y[1])/2;
  forehead_pos.z = (body->z[0] + body->z[1])/2;

  dx = forehead_pos.x - body->x[3]; // 3 is cap back marker
  dy = forehead_pos.y - body->y[3];
  dz = forehead_pos.z - body->z[3];
  dl = sqrt(dx*dx+dy*dy+dz*dz);

  /* Drawing the uncalibrated axis of FOV, which is directly based on markers position */

  *yaw = asin(dy/dl);

  *yaw -= YAW_CALIBRATION; // if no calibration has been done make sure YAW_CALIBRATION is set to 0. It will also necessary for finding the correct value of YAW_CALIBRATION because the calibration function will also call this function to find the calibration value.

  CURRENT_YAW = *yaw;

  if(dx<0&&dy>0) {
    *yaw=M_PI-(*yaw);
  }
  else {
    if(dx<0&&dy<0) {
      *yaw=-M_PI-(*yaw);
    }
  }

  *pitch = asin(dz/dl);

  *pitch -= PITCH_CALIBRATION; // // if no calibration has been done make sure PITCH_CALIBRATION is set to 0. It will also necessary for finding the correct value of PITCH_CALIBRATION because the calibration function will also call this function to find the calibration value.

  CURRENT_PITCH = *pitch;

  // THESE ARE CONSTANTS TO TRANSLATE THE MOCAP POSITION TO THE NECK OF THE HUMAN

  *x = forehead_pos.x;
  *y = forehead_pos.y;
  *z = forehead_pos.z-0.15; //TODO: best is to link this to Achile model

  return TRUE;

}


int mocap_compute_palm_position_from_glove(MOCAP_MARKERS_OBJECT *body, double *x, double *y, double *z)
{
  int i;

  *x = *y = *z = 0;

  if(body->nmarkers > 0) { // if there is at least one marker
    for(i=0; i<body->nmarkers; i++) {
      *x += body->x[i];
      *y += body->y[i];
      *z += body->z[i];
    }

    *x = *x/i;
    *y = *y/i;
    *z = *z/i;

    return TRUE;
  }
  else {
    return FALSE;
  }
}


int mocap_compute_neck_position_from_glasses(MOCAP_MARKERS_OBJECT *body, double *x, double *y, double *z, double *yaw, double *pitch)
{
  point_coords forehead_pos;
  double dx, dy, dz, dl;

  if(body == NULL)
    return FALSE;

  forehead_pos.x = (body->x[0] + body->x[1])/2; //0 is Left front 1 is right front markers
  forehead_pos.y = (body->y[0] + body->y[1])/2;
  forehead_pos.z = (body->z[0] + body->z[1])/2;

  dx = forehead_pos.x - (body->x[0]+body->x[2])/2.0; // 2 is right back marker
  dy = forehead_pos.y - (body->y[0]+body->y[2])/2.0;;
  dz = forehead_pos.z - (body->z[0]+body->z[2])/2.0;;
  dl = sqrt(dx*dx+dy*dy+dz*dz);


  /* Drawing the uncalibrated axis of FOV, which is directly based on markers position */

  *yaw = asin(dy/dl);

  *yaw -= YAW_CALIBRATION; // if no calibration has been done make sure YAW_CALIBRATION is set to 0. It will also necessary for finding the correct value of YAW_CALIBRATION because the calibration function will also call this function to find the calibration value.

  if( (dx<0) && (dy>0) ) {
    *yaw=M_PI-(*yaw);
  }
  else {
    if( (dx<0) && (dy<0) ) {
      *yaw=-M_PI-(*yaw);
    }
  }

  *pitch = asin(dz/dl);

  *pitch -= PITCH_CALIBRATION; // // if no calibration has been done make sure PITCH_CALIBRATION is set to 0. It will also necessary for finding the correct value of PITCH_CALIBRATION because the calibration function will also call this function to find the calibration value.


  // THESE ARE CONSTANTS TO TRANSLATE THE MOCAP POSITION TO THE NECK OF THE HUMAN

  *x = forehead_pos.x;
  *y = forehead_pos.y;
  *z = forehead_pos.z-0.1; //TODO: best is to link this to Achile model

  return TRUE;

}


#endif
