/*****************************************************************************
 *
 * Emrah Akin Sisbot
 * Aug 10
 *
 * File : sparkOro.h
 * Brief: This file contains functions for SPARK - Ontology link
 *
 ******************************************************************************/


#ifdef ORO

#include "server/sparkHeader.h"
#include "sparkm3d.h"

#include <string>
#include <map>
#include <iostream>
#include <stdlib.h>
#include <portLib.h>
#include <set>

// These are the common includes for the ontology:
#include <liboro/oro.h>
#include <liboro/oro_exceptions.h>
#include <liboro/socket_connector.h>

#if defined(USE_MIGHTABILITY_MAPS)
agents_activity_facts agActivityFactInOro[MAXI_NUM_OF_AGENT_FOR_HRI_TASK];
int agActivityFactInOroInitialized = FALSE;
int agActivityFactInOroAgentInitialized[MAXI_NUM_OF_AGENT_FOR_HRI_TASK];
#endif
int human_posture_oro = -1; //Value of agent human state in ontology.
//Hack right now. It should be managed better. It works only for one human.

using namespace std;
using namespace oro;

static SocketConnector *connector = NULL;
static Ontology *onto; //the accessor to the ontology singleton.

typedef struct agent_stmt {
  string agentID;
  set<Statement> stmtsToUpdate;
  set<string> stmtsToClear;
  set<Statement> stmtsToRemove;
  set<Statement> stmtsToAdd;
} agent_stmt;

vector<agent_stmt> agents_stmts;

static void addFactOperationInPoster(SPARK_FACT_OPERATION_TYPE type, char* agentModel ,char* subject, char* predicate, char* value)
{ 
  int curLineIndex = SDI_F->factOperations.curLineIndex;
  curLineIndex++;
  if(curLineIndex == SPARK_FACT_OPERATION_MAX){
    curLineIndex = 0;
    SDI_F->factOperations.curFullTableIter++;
  }

  SDI_F->factOperations.factOperation[curLineIndex].type = type;
  strcpy(SDI_F->factOperations.factOperation[curLineIndex].agentModel.name,agentModel);
  strcpy(SDI_F->factOperations.factOperation[curLineIndex].subject.name,subject);
  strcpy(SDI_F->factOperations.factOperation[curLineIndex].predicate.name,predicate);
  strcpy(SDI_F->factOperations.factOperation[curLineIndex].value.name,value);
  
  SDI_F->factOperations.curLineIndex = curLineIndex;  
}

int connectOro(char *host, char *port)
{

  //initialize the connection to the ontology server.
  string oro_server_host(host);
  string oro_server_port(port);
  time_t sendTime;
  time(&sendTime);

  try {

    if (connector == NULL) {
      connector = new SocketConnector(oro_server_host, oro_server_port);
      onto = Ontology::createWithConnector(*connector);
      // We don't wait for oro-server acknowledgement when sending fact.
      // Much faster, but we won't know if something goes wrong.
      onto->alwaysWaitForAcknowledgment(false);
      printf("[INFO] Connected to the Ontology Server!\n");
    }
    else {
      connector->reconnect(oro_server_host, oro_server_port);
      cout << "[INFO] " << ctime(&sendTime) << " : Connection with oro-server restablished!" << endl;
    }
  } catch (ConnectorException ce) {
    // Connection problem
    return -1;
  } catch (OntologyServerException ose) {
    // Server problem
    return -2;
  }
  return 1;
}

void clearFacts() {

  for (unsigned int i = 0; i < agents_stmts.size() ; i++) {
    agents_stmts[i].stmtsToClear.clear();
    agents_stmts[i].stmtsToRemove.clear();
    agents_stmts[i].stmtsToAdd.clear();
    agents_stmts[i].stmtsToUpdate.clear();
  }

}

/** Compute and format a millisecond-accurate timestamp **/
void timestamp(char* logtime)
{
	struct timeval  tv;
	struct timezone tz;
	struct tm      *tm;

	gettimeofday(&tv, &tz);
	tm = localtime(&tv.tv_sec);

	sprintf(logtime, "%04d%02d%02d %d:%02d:%02d.%03d", 
			    tm->tm_year + 1900,
			    tm->tm_mon,
			    tm->tm_mday,
				tm->tm_hour,
			    tm->tm_min, 
				tm->tm_sec, 
				((int)tv.tv_usec)/1000);
}

int sendToORO(GEN_STRING64 address, GEN_STRING64 port)
{
  unsigned int i;
  set<string>::iterator it;
  set<Statement>::iterator itu;

  //time_t sendTime;
  char logtime[80];
  //timestamp(logtime);

  try {
    if(!SDI_F->state.oroConnectionAv) {
      if ( connectOro(address.name, port.name) == 1 ) {
	SDI_F->state.oroConnectionAv = TRUE;
      }
      else {
	return FALSE;
      }
    }

    //cout << "TIME:" << time(0) << endl;
    for ( i = 0; i < agents_stmts.size() ; i++) {
      //cout << "FOR AGENT " << agents_stmts[i].agentID << ":" << endl;
      if ( !agents_stmts[i].stmtsToClear.empty()) {
	if(SDI_F->state.oroPrintEnabled){
	  // time(&sendTime);
	  //cout << "TIME:" << ctime(&sendTime) << endl;
	  timestamp(logtime);
	  cout << "TIME:" << logtime << endl;
	  cout << "\tCLEAR for agent " << agents_stmts[i].agentID << " contains:" << endl;
	  for ( it=agents_stmts[i].stmtsToClear.begin() ; it != agents_stmts[i].stmtsToClear.end(); it++ )
	    cout << "\t " << *it << endl;
	  cout << endl;
	}
	//-------------
	onto->clearForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToClear);
	//-------------
      }

      if ( !agents_stmts[i].stmtsToRemove.empty()) {
	if(SDI_F->state.oroPrintEnabled){
	  // time(&sendTime);
	  //cout << "TIME:" << ctime(&sendTime) << endl;
	  timestamp(logtime);
	  cout << "TIME:" << logtime << endl;
	  cout << "\tRemove " << agents_stmts[i].agentID << " contains:" << endl;
	  for ( itu=agents_stmts[i].stmtsToRemove.begin() ; itu != agents_stmts[i].stmtsToRemove.end(); itu++ )
	    cout << "\t " << *itu << endl;
	  cout << endl;
	}
	//-------------
	onto->removeForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToRemove);
	//-------------
      }

      if ( !agents_stmts[i].stmtsToAdd.empty()) {
	if(SDI_F->state.oroPrintEnabled){
	  // time(&sendTime);
	  // cout << "TIME:" << ctime(&sendTime) << endl;
	  timestamp(logtime);
	  cout << "TIME:" << logtime << endl;
	  cout << "\tADD " << agents_stmts[i].agentID <<  " contains:" <<endl;
	  for ( itu=agents_stmts[i].stmtsToAdd.begin() ; itu != agents_stmts[i].stmtsToAdd.end(); itu++ )
	    cout << "\t " << *itu << endl;
	  cout << endl;
	}
	//------------
	onto->addForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToAdd);
	//------------
      }

      if ( !agents_stmts[i].stmtsToUpdate.empty()) {
	if(SDI_F->state.oroPrintEnabled){
	  // time(&sendTime);
	  //cout << "TIME:" << ctime(&sendTime) << endl;
	  timestamp(logtime);
	  cout << "TIME:" << logtime << endl;
	  cout << "\tUPDATE " << agents_stmts[i].agentID <<  " contains:" <<endl;
	  for ( itu=agents_stmts[i].stmtsToUpdate.begin() ; itu != agents_stmts[i].stmtsToUpdate.end(); itu++ )
	    cout << "\t " << *itu << endl;
	  cout << endl;
	}
	//-----------
	onto->updateForAgent(agents_stmts[i].agentID, agents_stmts[i].stmtsToUpdate);
	//-----------
      }
    }


  } catch (const ConnectorException& ce) {
    // Connection problem
    printf("[WARNING] Disconnected from oro-server: %s.\n I will automatically reconnect when the server is up again.\n",ce.what());
    SDI_F->state.oroConnectionAv = FALSE;
    return FALSE;
  } catch (const OntologyServerException& ose) {
    // Server problem, but connection is ok.
    printf("[WARNING] Something went wrong while sending facts to oro-server: %s\n",ose.what());
    return FALSE;
  }
  return TRUE;
}


/** Creates the set of symbolic facts for later exportation to the knowledge base
 *
 * Modification added by Matthieu Warnier 17/01/2011:
 * To avoid sending information forever, I have included the is_changed and
 * is_exported bool. Information is sent only if we have both is_changed TRUE
 * ( value changed since the last computation ) and is_exported FALSE
 * ( It was not yet exported. ) is_exported is put to TRUE after export
 */
void prepareFactsForORO(HRI_AGENTS * agents, HRI_ENTITIES * ents)
{
    char tempBuffer[100];
    int a_i,a_j, e_i, e_j;
    agent_stmt AS;
    HRI_KNOWLEDGE_ON_ENTITY * kn_on_ent;
    HRI_AGENT * agent;
    HRI_AGENT * agent2;

    // For testing purpose we populate this temporaraly deprecated poster with Symbolic Placement values.
    //bool needToExportObjStatePoster = false;
    //bool useObjStatePoster = true;
   
    bool reexportReachability = false; // for reachability Spark / reachability Motion planning conflict management we reexport reachability.

    /** TODO: hack to force reexport of all. (especially for appear or 
     * disappear management) It should definitely be improved this. We 
     * should have a structure that has all the values that were exported 
     * to the ontology to compare directly to it.
     */
    bool reexportAll = false;
    
    ///Use by DistanceInfo export
    HRI_DISTANCE_AND_SPEED_INFO* distSpeedInfo;
    char* agentPartName;
    double distanceScoreStep;
    bool createScore,deleteScore;
    int dist_i,agentPartIndex,intDistanceScoreOld,intDistanceScoreNew;    

    //to monitor how many facts operations were added in this function. We save initial values
    int factOperationsCurLineIndexOld = SDI_F->factOperations.curLineIndex;
    int factOperationsCurFullTableIteOld = SDI_F->factOperations.curFullTableIter;

    // initialization of sets
    if ( agents_stmts.empty() ) {
        for(a_i=0; a_i<agents->all_agents_no; a_i++){
            AS.agentID = string(agents->all_agents[a_i]->robotPt->name);
            agents_stmts.push_back(AS);
        }
    }

    // store the robot own index. Used to save all agent specific statements in the
    // robot model as well.
    int robot_idx = agents->source_agent_idx;

    for(a_i=0; a_i<agents->all_agents_no; a_i++) {

      agent = agents->all_agents[a_i];

      // If the agent is in the scene
      if(!agent->is_present) continue;

      //////////////////////////////////////////////////////
      /// isSitting / isStanding provisiary management.  ///
      //////////////////////////////////////////////////////


      /// Be Careful : It works only for one agent.
      /// If Agent is Robot. Export isSitting or isStanding for one other agent. The first present that is different from robot. It should be managed better in the future.
      if(a_i == agents->source_agent_idx){
        for(a_j=0; a_j<agents->all_agents_no; a_j++) {
          if(a_i != a_j){
            agent2 = agents->all_agents[a_j];
            if(agent2->is_present)
            break;
          }
        }

        if(human_posture_oro != agent2->actual_state){

          switch(agent2->actual_state) {
          case 0:
            sprintf(tempBuffer,"%s hasPosture sitting",agent2->robotPt->name);
            agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	        addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,agent2->robotPt->name, (char*) "hasPosture", (char*) "sitting");
            break;
          case 1:
            sprintf(tempBuffer,"%s hasPosture standing",agent2->robotPt->name);
            agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
	        addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,agent2->robotPt->name, (char*) "hasPosture", (char*) "standing");
            break;
          }
          human_posture_oro = agent2->actual_state;
        }
      }

      for(e_i=0; e_i<agent->knowledge->entities_nb; e_i++) {

        kn_on_ent = &agent->knowledge->entities[e_i];

        // If the entity is a part of the current agent, we skip it since it doesn't make sense to compute it from his own point of view
        // TODO: Or does it?
        if( (kn_on_ent->entPt->type == HRI_AGENT_PART) || (kn_on_ent->entPt->type == HRI_ISAGENT) ) {
            if( agent == agents->all_agents[kn_on_ent->entPt->agent_idx] )
                continue;
        }

        // If the entity is present in the scene
        if(kn_on_ent->entPt->is_present) {

        // Export presence. Only for agent now;
        if( (kn_on_ent->entPt->type == HRI_ISAGENT) && (!kn_on_ent->presence_isexported || !kn_on_ent->presenceValueExported)) {
            sprintf(tempBuffer,"%s isPresent true", kn_on_ent->entPt->name);
            agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
            addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isPresent", (char*) "true");

            // Store this fact in the robot model as well
            if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToAdd.insert(Statement(tempBuffer));

            kn_on_ent->presenceValueExported = TRUE;
            kn_on_ent->presence_isexported = TRUE;
        }

        // Check if the entity has disappeared
        if(kn_on_ent->entPt->disappeared) {
            if( !kn_on_ent->disappeared_isexported) {
                reexportAll = true;
		        sprintf(tempBuffer,"%s isAt unknownLocation", kn_on_ent->entPt->name);		
                agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
                addFactOperationInPoster(SPARK_UPDATE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isAt", (char*) "unknownLocation");
                //
                kn_on_ent->disappeared_isexported = TRUE;
            }
        }

        // delete isAt unknown location
        if( !kn_on_ent->disappeared_isexported) {
            reexportAll = true;
            sprintf(tempBuffer,"%s isAt ?val",kn_on_ent->entPt->name);
            agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
            addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isAt", (char*) "whatEverValue");
            kn_on_ent->disappeared_isexported = TRUE;
        }

	///Divergent belief Management for known unknown positions.
	if(ents->manageDivergentBeliefs && ( kn_on_ent->entPt->subtype == HRI_MOVABLE_OBJECT)){
	  if (kn_on_ent->hasEntityPositionKnowledge != kn_on_ent->hasEntityPositionKnowledgeExportedValue){
		kn_on_ent->hasEntityPositionKnowledgeExportedValue = kn_on_ent->hasEntityPositionKnowledge;
	    if(kn_on_ent->hasEntityPositionKnowledge){
	      sprintf(tempBuffer,"%s hasKnownLocation true", kn_on_ent->entPt->name);
	      //printf("%s hasKnownLocation true\n", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name,(char*) "hasKnownLocation", (char*) "true");
	      sprintf(tempBuffer,"%s hasKnownLocation false",kn_on_ent->entPt->name);
	      //printf("%s hasKnownLocation false",kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name,(char*) "hasKnownLocation",(char*) "false");
	      SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForDivergentBelief(a_i,e_i,false,true);	  	    
	    }
	    else{
	      sprintf(tempBuffer,"%s hasKnownLocation false",kn_on_ent->entPt->name);
	      //printf("%s hasKnownLocation false\n",kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasKnownLocation", (char*) "false");
	      sprintf(tempBuffer,"%s hasKnownLocation true", kn_on_ent->entPt->name);
	      //printf("%s hasKnownLocation true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasKnownLocation", (char*) "true");
	      /// it will be repeated below as now hasDistinctPosition become false when hasKnownLocation become true.
	      /// we keep it like this in case of future changes to this behaviour.
	      SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForDivergentBelief(a_i,e_i,false,false);
	    }
	  }
	  if (kn_on_ent->hasEntityPosition != kn_on_ent->hasEntityPositionExportedValue){
	    kn_on_ent->hasEntityPositionExportedValue = kn_on_ent->hasEntityPosition;
		if(kn_on_ent->hasEntityPosition){
	      sprintf(tempBuffer,"%s hasDistinctLocation true", kn_on_ent->entPt->name);
	      //printf("%s hasDistinctLocation true\n", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasDistinctLocation", (char*) "true");
	      sprintf(tempBuffer,"%s hasDistinctLocation false",kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasDistinctLocation", (char*) "false");
	      SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForDivergentBelief(a_i,e_i,true,true);
	    }
	    else{
	      sprintf(tempBuffer,"%s hasDistinctLocation false",kn_on_ent->entPt->name);
	      //printf("%s hasDistinctLocation false\n",kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasDistinctLocation", (char*) "false");
	      sprintf(tempBuffer,"%s hasDistinctLocation true", kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
	      addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "hasDistinctLocation", (char*) "true");
	      if(kn_on_ent->hasEntityPositionKnowledge)
		SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForDivergentBelief(a_i,e_i,false,true);
	      else
		SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForDivergentBelief(a_i,e_i,false,false);
	    }
	  }
	}

    // Static - Moving
    if ( (kn_on_ent->motion_ischanged && !kn_on_ent->motion_isexported) || reexportAll) {
       switch(kn_on_ent->motion) {
         case HRI_MOVING:
            sprintf(tempBuffer,"%s isMoving true", kn_on_ent->entPt->name);
            agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
            addFactOperationInPoster(SPARK_UPDATE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isMoving", (char*) "true");
            break;
         case HRI_STATIC:
            sprintf(tempBuffer,"%s isMoving false", kn_on_ent->entPt->name);
            agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
            addFactOperationInPoster(SPARK_UPDATE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isMoving", (char*) "false");
            //We reexport reachability if object has moved and stop moving
            //Doing this we ensure that reachability mhp will be replace by reachability spark
            //Actually reachability spark is not yet updated but if it change when updated it will be reexported
            //and if it didn't changed it will be this value.
            if(kn_on_ent->motion_ischanged && !kn_on_ent->motion_isexported)
		      if(SDI_F->state.reexportReachabilityFactsToOro){
			reexportReachability = true;
			SDI_F->state.reexportReachabilityFactsToOro = FALSE;
		      }
            break;
         case HRI_UK_MOTION:
            sprintf(tempBuffer,"%s isMoving ?val", kn_on_ent->entPt->name);
            agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
            addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isMoving", (char*) "whatEverValue");
            break;
       }
       kn_on_ent->motion_isexported = TRUE;
    }

	// Distance Info Export
	// Compute Distance Score between some Agent Part and Object.
	if (kn_on_ent->manageDistanceAndSpeedInfoArraySize > 0){
	    for(dist_i=0; dist_i<kn_on_ent->manageDistanceAndSpeedInfoArraySize; dist_i++) {
		agentPartIndex = kn_on_ent->manageDistanceAndSpeedInfoArrayAgentPartIndex[dist_i];
		createScore = false;
		deleteScore = false;
		intDistanceScoreNew = 1000;    
		intDistanceScoreOld = 1000;
		if(agentPartIndex == -1){
		    agentPartName = agent->robotPt->name;
		    distanceScoreStep = 0.3;
		}
		else{
		    agentPartName = agent->hand[agentPartIndex]->name;
		    distanceScoreStep = 0.07;
		}

		distSpeedInfo = &kn_on_ent->distanceAndSpeedInfoArray[dist_i];

		if(distSpeedInfo->averageDistanceScoreOld > ents->averageDistanceScoreExportThreshold){
		    intDistanceScoreOld = distSpeedInfo->averageDistanceScoreOld/distanceScoreStep;
		    deleteScore = true;
		}
		
		if(distSpeedInfo->averageDistanceScore > ents->averageDistanceScoreExportThreshold){
		    intDistanceScoreNew = distSpeedInfo->averageDistanceScore/distanceScoreStep;
		    if( deleteScore && (intDistanceScoreNew == intDistanceScoreOld))
			deleteScore = false;
		    else if(!deleteScore || (deleteScore && (intDistanceScoreNew != intDistanceScoreOld)))
			createScore = true;
		}		 
		if(deleteScore){
		    sprintf(tempBuffer,"%s%s hasDistanceScoreOf %d", agentPartName , kn_on_ent->entPt->name , intDistanceScoreOld);
		    agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
		    //addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name,"hasDistanceScoreOf", intDistanceScoreOld.toString());
		}
		if(createScore){
		    sprintf(tempBuffer,"%s%s hasDistanceScoreOf %d", agentPartName , kn_on_ent->entPt->name , intDistanceScoreOld);
		    agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
		    //addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name,"hasDistanceScoreOf",intDistanceScoreOld.toString());
		}
	  }
	}
  
        // Visibile - Invisible
        if ( (kn_on_ent->visibility_ischanged && !kn_on_ent->visibility_isexported) || reexportAll) {
            switch(kn_on_ent->visibility) {
                case HRI_VISIBLE:
                    sprintf(tempBuffer,"%s isVisible true", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
		            addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isVisible", (char*) "true");
                    break;
                case HRI_INVISIBLE:
                    sprintf(tempBuffer,"%s isVisible false", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
		            addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isVisible", (char*) "false");
                    break;
                case HRI_UK_VIS:
                    sprintf(tempBuffer,"%s isVisible ?val", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
		            addFactOperationInPoster(SPARK_CLEAR_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isVisible", (char*) "whatEverValue");
                    break;
            }
            kn_on_ent->visibility_isexported = TRUE;
        }

        // Reachability - Commented out for performance reasons
        // TODO: Make a difference in ORO for HARDLY_REACHABLE
        if ((kn_on_ent->reachability_ischanged && !kn_on_ent->reachability_isexported) || reexportReachability  || reexportAll) {
            switch(kn_on_ent->reachability) {
                case HRI_REACHABLE:
                    sprintf(tempBuffer,"%s isReachable true", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
                    addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isReachable", (char*) "true");
		            SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForReachability(a_i,e_i,true);
                    break;
                case HRI_HARDLY_REACHABLE:
                    sprintf(tempBuffer,"%s isReachable true", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
                    addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isReachable", (char*) "true");
		            SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForReachability(a_i,e_i,true);
                    break;
                case HRI_UNREACHABLE:
                    sprintf(tempBuffer,"%s isReachable false", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToUpdate.insert(Statement(tempBuffer));
                    addFactOperationInPoster(SPARK_UPDATE_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isReachable", (char*) "false");
		            SDI_F->visuInfoManagement.numVisuUpdate+=UpdateSomeVisuInfoForReachability(a_i,e_i,false);
                    break;
                case HRI_UK_REACHABILITY:
                    sprintf(tempBuffer,"%s isReachable ?val", kn_on_ent->entPt->name);
                    agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
                    addFactOperationInPoster(SPARK_CLEAR_FACT,agent->robotPt->name,kn_on_ent->entPt->name, (char*) "isReachable", (char*) "whatEverValue");
                    break;
            }
            kn_on_ent->reachability_isexported = TRUE;
        }

        // FOA - FOV - OOF
        // TODO: Make ORO accept FOV-FOZ-OOF information

	
	if(!SDI_F->state.inhibitLookDirectionOroFactsSending){
	  // LOOKSAT
	  if ( (kn_on_ent->is_looked_at_ischanged && !kn_on_ent->is_looked_at_isexported) || reexportAll) {
            if(kn_on_ent->is_looked_at == HRI_TRUE_V) {
	          sprintf(tempBuffer,"%s looksAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	          agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	          addFactOperationInPoster(SPARK_ADD_FACT,agent->robotPt->name, agent->robotPt->name, (char*) "looksAt", kn_on_ent->entPt->name);

	          // Store this fact in the robot model as well
	          if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToAdd.insert(Statement(tempBuffer));
            }else {
	          sprintf(tempBuffer,"%s looksAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	          agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
	          addFactOperationInPoster(SPARK_REMOVE_FACT,agent->robotPt->name, agent->robotPt->name, (char*) "looksAt", kn_on_ent->entPt->name);


	      // Remove this fact from the robot model as well
	      if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToRemove.insert(Statement(tempBuffer));
            }
            kn_on_ent->is_looked_at_isexported = TRUE;
	  }

	  // ISINFIELDOFVIEW
	  if ( (kn_on_ent->isSeenischanged && !kn_on_ent->isSeenisexported) || reexportAll) {
            if(kn_on_ent->isSeen == HRI_TRUE_V) {
	      sprintf(tempBuffer,"%s sees %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, agent->robotPt->name, (char*) "sees", kn_on_ent->entPt->name);


	      // Store this fact in the robot model as well
	      if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToAdd.insert(Statement(tempBuffer));
            }
            else {
	      sprintf(tempBuffer,"%s sees %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, agent->robotPt->name, (char*) "sees", kn_on_ent->entPt->name);

	      // Remove this fact from the robot model as well
	      if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToRemove.insert(Statement(tempBuffer));
            }
            kn_on_ent->isSeenisexported = TRUE;
	  }

	  // POINTSAT
	  if ( (kn_on_ent->is_pointed_at_ischanged && !kn_on_ent->is_pointed_at_isexported) || reexportAll) {
            if(kn_on_ent->is_pointed_at == HRI_TRUE_V) {
	      sprintf(tempBuffer,"%s pointsAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, agent->robotPt->name, (char*) "pointsAt", kn_on_ent->entPt->name);

	      // Store this fact in the robot model as well
	      if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToAdd.insert(Statement(tempBuffer));
            }
            else {
	      sprintf(tempBuffer,"%s pointsAt %s", agent->robotPt->name, kn_on_ent->entPt->name);
	      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
	      addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, agent->robotPt->name, (char*) "sees", kn_on_ent->entPt->name);

	      // Remove this fact from the robot model as well
	      if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToRemove.insert(Statement(tempBuffer));
            }
            kn_on_ent->is_pointed_at_isexported = TRUE;
	  }
	}

	///Divergent belief Management.
	if(ents->manageDivergentBeliefs){
	    for(a_j=0; a_j<agents->all_agents_no; a_j++) {
		agent2 = agents->all_agents[a_j];
		// Visibile - Invisible By
		if ( (kn_on_ent->visibilityBy_ischanged[a_j] && !kn_on_ent->visibilityBy_isexported[a_j]) || reexportAll) {
		    switch(kn_on_ent->visibilityBy[a_j]) {
		    case HRI_VISIBLE:
			// sprintf(tempBuffer,"%s isNotVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			// addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotVisibleBy", agent2->robotPt->name);
			sprintf(tempBuffer,"%s isVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isVisibleBy", agent2->robotPt->name);
			break;
		    case HRI_INVISIBLE:
			sprintf(tempBuffer,"%s isVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isVisibleBy", agent2->robotPt->name);
			// sprintf(tempBuffer,"%s isNotVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotVisibleBy", agent2->robotPt->name);
			break;
		    case HRI_UK_VIS:
			sprintf(tempBuffer,"%s isVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isVisibleBy", agent2->robotPt->name);
			// sprintf(tempBuffer,"%s isNotVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			// addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotVisibleBy", agent2->robotPt->name);
			break;
		    }
		    kn_on_ent->visibilityBy_isexported[a_j] = TRUE;
		}
		// ReachabilityBy - Commented out for performance reasons
		// TODO: Make a difference in ORO for HARDLY_REACHABLE
		if ((kn_on_ent->reachabilityBy_ischanged[a_j] && !kn_on_ent->reachabilityBy_isexported[a_j]) || reexportReachability  || reexportAll) {
		    switch(kn_on_ent->reachabilityBy[a_j]) {
		    case HRI_REACHABLE:
			// sprintf(tempBuffer,"%s isNotReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			// addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotReachableBy", agent2->robotPt->name);
			sprintf(tempBuffer,"%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isReachableBy", agent2->robotPt->name);
			break;
		    case HRI_HARDLY_REACHABLE:
			sprintf(tempBuffer,"%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isReachableBy", agent2->robotPt->name);
			break;
		    case HRI_UNREACHABLE:
			sprintf(tempBuffer,"%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isReachableBy", agent2->robotPt->name);
			// sprintf(tempBuffer,"%s isNotReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotReachableBy", agent2->robotPt->name);
			break;
		    case HRI_UK_REACHABILITY:
			sprintf(tempBuffer,"%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isReachableBy", agent2->robotPt->name);
			// sprintf(tempBuffer,"%s isNotReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
			// agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
			// addFactOperationInPoster(SPARK_CLEAR_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNotReachableBy", agent2->robotPt->name);
			break;
		    }
		    kn_on_ent->reachabilityBy_isexported[a_j] = TRUE;
		}

		
		if(!SDI_F->state.inhibitLookDirectionOroFactsSending){
		  // LOOKSATBY
		  if ( (kn_on_ent->is_looked_atBy_ischanged[a_j] && !kn_on_ent->is_looked_atBy_isexported[a_j]) || reexportAll) {
		    if(kn_on_ent->is_looked_atBy[a_j] == HRI_TRUE_V) {
		      sprintf(tempBuffer,"%s isLookedAtBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isLookedAtBy", agent2->robotPt->name);
		    }
		    else {
		      sprintf(tempBuffer,"%s isLookedAtBy %s", kn_on_ent->entPt->name, agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isLookedAtBy", agent2->robotPt->name);
		    }
		    kn_on_ent->is_looked_atBy_isexported[a_j] = TRUE;
		  }

		  // ISINFIELDOFVIEWBY
		  if ( (kn_on_ent->isSeenByischanged[a_j] && !kn_on_ent->isSeenByisexported[a_j]) || reexportAll) {
		    if(kn_on_ent->isSeenBy[a_j] == HRI_TRUE_V) {
		      sprintf(tempBuffer,"%s isSeenBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isSeenBy", agent2->robotPt->name);
		    }
		    else {
		      sprintf(tempBuffer,"%s isSeenBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isSeenBy", agent2->robotPt->name);
		    }
		    kn_on_ent->isSeenByisexported[a_j] = TRUE;
		  }

		  // POINTSATBY
		  if ( (kn_on_ent->is_pointed_atBy_ischanged[a_j] && !kn_on_ent->is_pointed_atBy_isexported[a_j]) || reexportAll) {
		    if(kn_on_ent->is_pointed_atBy[a_j] == HRI_TRUE_V) {
		      sprintf(tempBuffer,"%s isPointedAtBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isPointedAtBy", agent2->robotPt->name);
		    }
		    else {
		      sprintf(tempBuffer,"%s isPointedAtBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			  addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isPointedAtBy", agent2->robotPt->name);
		    }
		    kn_on_ent->is_pointed_atBy_isexported[a_j] = TRUE;
		  }
		}
	    }
	}
	      
        // FRONT - BACK - ETC
        // if ( (kn_on_ent->spatial_relation_ischanged && !kn_on_ent->spatial_relation_isexported) || reexportAll) {
        //   if(kn_on_ent->is_left_right_from_agent == HRI_LEFT) {
        //     sprintf(tempBuffer,"%s leftOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s rightOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //   }
        //   else if(kn_on_ent->is_left_right_from_agent == HRI_RIGHT) {
        //     sprintf(tempBuffer,"%s rightOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s leftOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }
        //   else {
        //     sprintf(tempBuffer,"%s rightOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //     sprintf(tempBuffer,"%s leftOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }

        //   if(kn_on_ent->is_front_behind_from_agent == HRI_FRONT) {
        //     sprintf(tempBuffer,"%s inFrontOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s behind %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //   }
        //   else if(kn_on_ent->is_front_behind_from_agent == HRI_BEHIND) {
        //     sprintf(tempBuffer,"%s behind %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s inFrontOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }
        //   else {
        //     sprintf(tempBuffer,"%s behind %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //     sprintf(tempBuffer,"%s inFrontOf %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }

        //   if(kn_on_ent->is_far_near_from_agent == HRI_FAR) {
        //     sprintf(tempBuffer,"%s cyc:farFrom %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s cyc:near %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //   }
        //   else if(kn_on_ent->is_far_near_from_agent == HRI_NEAR) {
        //     sprintf(tempBuffer,"%s cyc:near %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
        //     sprintf(tempBuffer,"%s cyc:farFrom %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }
        //   else {
        //     sprintf(tempBuffer,"%s cyc:farFrom %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);
        //     sprintf(tempBuffer,"%s cyc:near %s", kn_on_ent->entPt->name , agent->robotPt->name);
        //     agents_stmts[a_i].stmtsToClear.insert(tempBuffer);			
        //   }
        //
        //   kn_on_ent->spatial_relation_isexported = TRUE;
        // }

        // IN - ON - NEXTTO

        for(e_j=0; e_j<kn_on_ent->is_placed_nb; e_j++) {
            // placement relation between the same object has no meaning
            if ( ( e_j != e_i) && ents->entities[e_j]->is_present && ((kn_on_ent->placement_relation_ischanged[e_j] && !kn_on_ent->placement_relation_isexported[e_j]) || reexportAll)) {
                // delete previous value. ( right now only on placement relation is possible )
                switch(kn_on_ent->is_placed_old[e_j]){
                case HRI_UK_PLR:
                    //Nothing to do
                    break;
                case HRI_ISON:
                    sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                    agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		    addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isOn", ents->entities[e_j]->name);
                    break;
                case HRI_ISNEXTTO:
                    sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                    agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		    addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isNextTo", ents->entities[e_j]->name);
                    break;
                case HRI_ISIN:
                    sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                    agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		    addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isIn", ents->entities[e_j]->name);
                    break;
                case HRI_NOPLR:
                    break;
                }

                // Populate Poster to test and manage oro event delay on isOn

                switch(kn_on_ent->is_placed[e_j]) {
                    case HRI_ISIN:
                        // sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isOn", ents->entities[e_j]->name);

                        // sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNextTo", ents->entities[e_j]->name);

                        sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isIn", ents->entities[e_j]->name);

                        break;
                    case HRI_ISON:
                        // sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isNextTo", ents->entities[e_j]->name);
                        
			// sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isIn", ents->entities[e_j]->name);

                        sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isOn", ents->entities[e_j]->name);
                        break;
                    case HRI_ISNEXTTO:
                        // sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isOn", ents->entities[e_j]->name);
                        
			// sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isIn", ents->entities[e_j]->name);

                        sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        agents_stmts[a_i].stmtsToAdd.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_ADD_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isNextTo", ents->entities[e_j]->name);
                        break;
                    case HRI_UK_PLR:
                        // sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isOn", ents->entities[e_j]->name);

                        // sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);                        
			// agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isNextTo", ents->entities[e_j]->name);

                        sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isIn", ents->entities[e_j]->name);
                        
			break;
                    case HRI_NOPLR:
                        // sprintf(tempBuffer,"%s isOn %s", kn_on_ent->entPt->name,ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isOn", ents->entities[e_j]->name);
                        
			// sprintf(tempBuffer,"%s isNextTo %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isNextTo", ents->entities[e_j]->name);
                        
			// sprintf(tempBuffer,"%s isIn %s", kn_on_ent->entPt->name, ents->entities[e_j]->name);
                        // agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
			// addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, "isIn", ents->entities[e_j]->name);
                        
			break;
                }
                kn_on_ent->placement_relation_isexported[e_j] = TRUE;
            }
            //reinit forced reexport for this entity.
            reexportReachability = false;
            reexportAll = false;

        }

        }
        else { // Object is not present
            // Remove presence. Only for agent now;
            if( (kn_on_ent->entPt->type == HRI_ISAGENT) && (!kn_on_ent->presence_isexported || kn_on_ent->presenceValueExported)) {
                sprintf(tempBuffer,"%s isPresent true", kn_on_ent->entPt->name);
                agents_stmts[a_i].stmtsToRemove.insert(Statement(tempBuffer));
		addFactOperationInPoster(SPARK_REMOVE_FACT, agent->robotPt->name, kn_on_ent->entPt->name, (char*) "isPresent", (char*) "true");

                // Remove this fact from the robot model as well
                if (a_i != robot_idx) agents_stmts[robot_idx].stmtsToRemove.insert(Statement(tempBuffer));

                kn_on_ent->presenceValueExported = FALSE;
                kn_on_ent->presence_isexported = TRUE;
            }
        }
	
      } // end for each entity
      
    } // end for each agent
    
    //fact operation poster
    int newFactOperatinsNum = (SDI_F->factOperations.curFullTableIter - factOperationsCurFullTableIteOld)*SPARK_FACT_OPERATION_MAX + SDI_F->factOperations.curLineIndex-factOperationsCurLineIndexOld;
    if(newFactOperatinsNum > SDI_F->factOperations.maxFactOperationsUpdateNum)
      SDI_F->factOperations.maxFactOperationsUpdateNum = newFactOperatinsNum;

    sparkFactOperationsfactOperationsPosterWrite(SPARK_FACTOPERATIONS_POSTER_ID, &SDI_F->factOperations);

    if(SDI_F->visuInfoManagement.numVisuUpdate > 0){
      publishVisuInfoPoster();
      SDI_F->visuInfoManagement.numVisuUpdate = 0;
    }
}
  
  

/** Exports to oro are only done for facts that are updated
 * In case of Oro restart we need to resend everything
 * This function reinits export control.
 */
void reinitFactsForOROExportControl(HRI_AGENTS * agents, HRI_ENTITIES * ents)
{

  int a_i, a_j,e_i, e_j;
  HRI_KNOWLEDGE_ON_ENTITY * kn_on_ent;
  HRI_AGENT * agent;

  ///Reinit for one human posture.
  human_posture_oro = -1;

  for(a_i=0; a_i<agents->all_agents_no; a_i++) {
    agent = agents->all_agents[a_i];

    // If the agent is in the scene
    if(agent->is_present) {

      for(e_i=0; e_i<agent->knowledge->entities_nb; e_i++) {
	kn_on_ent = &agent->knowledge->entities[e_i];

	// If the entity is a part of the current agent, we skip it since it doesn't make sense to compute it from his own point of view
	// TODO: Or does it?
	if( (kn_on_ent->entPt->type == HRI_AGENT_PART) || (kn_on_ent->entPt->type == HRI_ISAGENT) ) {
	  if( agent == agents->all_agents[kn_on_ent->entPt->agent_idx] )
	    continue;
	}

	// If the entity is present in the scene
	if(kn_on_ent->entPt->is_present) {

	  // IsPresent control :
	  kn_on_ent->presence_isexported = FALSE;

	  // Check if the entity has disappeared
	  if(kn_on_ent->entPt->disappeared) {
	    kn_on_ent->disappeared_isexported = FALSE;
	  }

	  // Static - Moving
	  kn_on_ent->motion_ischanged = TRUE;
	  kn_on_ent->motion_isexported = FALSE;

	  // Visibile - Invisible
	  kn_on_ent->visibility_ischanged = TRUE;
	  kn_on_ent->visibility_isexported = FALSE;

	  // Reachability - Commented out for performance reasons
	  // TODO: Make a difference in ORO for HARDLY_REACHABLE
	  kn_on_ent->reachability_ischanged = TRUE;
	  kn_on_ent->reachability_isexported = FALSE;

	  // LOOKSAT
	  kn_on_ent->is_looked_at_ischanged = TRUE;
	  kn_on_ent->is_looked_at_isexported = FALSE;

	  // LOOKSAT
	  kn_on_ent->isSeenischanged = TRUE;
	  kn_on_ent->isSeenisexported = FALSE;

	  // POINTSAT
	  kn_on_ent->is_pointed_at_ischanged = TRUE;
	  kn_on_ent->is_pointed_at_isexported = FALSE;

	  if(ents->manageDivergentBeliefs){
	      for(a_j=0; a_j<agents->all_agents_no; a_j++) {
		  // If the agent is in the scene
		      kn_on_ent->visibility_placementBy_ischanged[a_j] = TRUE;
		      kn_on_ent->visibility_placementBy_isexported[a_j] = FALSE;
		  
		      kn_on_ent->visibilityBy_ischanged[a_j] = TRUE;
		      kn_on_ent->visibilityBy_isexported[a_j] = FALSE;

		      kn_on_ent->reachabilityBy_ischanged[a_j] = TRUE;
		      kn_on_ent->reachabilityBy_isexported[a_j] = FALSE;

		      kn_on_ent->is_looked_atBy_ischanged[a_j] = TRUE;
		      kn_on_ent->is_looked_atBy_isexported[a_j] = FALSE;

		      kn_on_ent->isSeenByischanged[a_j] = TRUE;
		      kn_on_ent->isSeenByisexported[a_j] = FALSE;

		      kn_on_ent->is_pointed_atBy_ischanged[a_j] = TRUE;
		      kn_on_ent->is_pointed_atBy_isexported[a_j] = FALSE;
		  }
	  }

	  // FRONT - BACK - ETC
	  kn_on_ent->spatial_relation_ischanged = TRUE;
	  kn_on_ent->spatial_relation_isexported = FALSE;

	  // IN - ON - NEXTTO

	  for(e_j=0; e_j<kn_on_ent->is_placed_nb; e_j++) {
	    // placement relation between the same object has no meaning
	    kn_on_ent->placement_relation_ischanged[e_j] = TRUE;
	    kn_on_ent->placement_relation_isexported[e_j] = FALSE;
	  }
	}
      }
    }
  }
}




//
//Matthieu Warnier 4/05/2011
//Print all
//
void printAllOroFacts(HRI_AGENTS * agents, HRI_ENTITIES * ents)
{
  int a_i,a_j, e_i, e_j;
  agent_stmt AS;
  HRI_KNOWLEDGE_ON_ENTITY * kn_on_ent;
  HRI_AGENT * agent;
  HRI_AGENT * agent2;

  // initialization of sets
  if ( agents_stmts.empty() ) {
    for(a_i=0; a_i<agents->all_agents_no; a_i++){
      AS.agentID = string(agents->all_agents[a_i]->robotPt->name);
      agents_stmts.push_back(AS);
    }
  }


  for(a_i=0; a_i<agents->all_agents_no; a_i++) {

    agent = agents->all_agents[a_i];

    // If the agent is in the scene
    if(agent->is_present) {

      printf("Prints of Oro Facts for Model of Agent %s\n", agent->robotPt->name);

      //////////////////////////////////////////////////////
      /// isSitting / isStanding provisiary management.  ///
      //////////////////////////////////////////////////////

      /// If Agent is Robot. Export isSitting or isStanding for one other agent. The first present that is different from robot. It should be managed better in the future.
      if(a_i == agents->source_agent_idx){
	for(a_j=0; a_j<agents->all_agents_no; a_j++) {
	  if(a_i != a_j){
	    agent2 = agents->all_agents[a_j];
	    if(agent2->is_present)
	      break;
	  }
	}
	switch(human_posture_oro) {
	case 0:
	  printf("%s isSitting true\n",agent2->robotPt->name);
	  break;
	case 1:
	  printf("%s isStanding true\n",agent2->robotPt->name);
	  break;
	}
      }

      ///Explore Agent Knowledge:
      for(e_i=0; e_i<agent->knowledge->entities_nb; e_i++) {

	kn_on_ent = &agent->knowledge->entities[e_i];

	// If the entity is a part of the current agent, we skip it since it doesn't make sense to compute it from his own point of view
	// TODO: Or does it?
	if( (kn_on_ent->entPt->type == HRI_AGENT_PART) || (kn_on_ent->entPt->type == HRI_ISAGENT) ) {
	  if( agent == agents->all_agents[kn_on_ent->entPt->agent_idx] )
	    continue;
	}

	// If the entity is present in the scene
	if(kn_on_ent->entPt->is_present) {

	  // Check if the entity has disappeared

	  if(kn_on_ent->entPt->disappeared) {
	    // update isAt unknownLocation
	    printf("%s isAt unknownLocation\n", kn_on_ent->entPt->name);
	  }

	  // Static - Moving
	  switch(kn_on_ent->motion) {
	  case HRI_MOVING:
	    printf("%s isMoving true\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_STATIC:
	    printf("%s isMoving false\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_UK_MOTION:
	    printf("%s isMoving unknown\n", kn_on_ent->entPt->name);
	    break;
	  }

	  // Visibile - Invisible
	  switch(kn_on_ent->visibility) {
	  case HRI_VISIBLE:
	    printf("%s isVisible true\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_INVISIBLE:
	    printf("%s isVisible false\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_UK_VIS:
	    printf("%s isVisible unknown\n", kn_on_ent->entPt->name);
	    break;
	  }

	  // Reachability - Commented out for performance reasons
	  // TODO: Make a difference in ORO for HARDLY_REACHABLE
	  switch(kn_on_ent->reachability) {
	  case HRI_REACHABLE:
	    printf("%s isReachable true\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_HARDLY_REACHABLE:
	    printf("%s isReachable true\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_UNREACHABLE:
	    printf("%s isReachable false\n", kn_on_ent->entPt->name);
	    break;
	  case HRI_UK_REACHABILITY:
	    printf("%s isReachable unknown\n", kn_on_ent->entPt->name);
	    break;
	  }

	  // LOOKSAT
	  if(kn_on_ent->is_looked_at == HRI_TRUE_V) {
	    printf("%s looksAt %s\n", agent->robotPt->name, kn_on_ent->entPt->name);
	  }

	  // ISINFIELDOFVIEW
	  if(kn_on_ent->isSeen == HRI_TRUE_V) {
	    printf("%s sees %s\n", agent->robotPt->name , kn_on_ent->entPt->name);
	  }

	  // POINTSAT
	  if(kn_on_ent->is_pointed_at == HRI_TRUE_V) {
	    printf("%s pointsAt %s\n", agent->robotPt->name, kn_on_ent->entPt->name);
	  }
 

	  ///Divergent belief Management.
	  if(ents->manageDivergentBeliefs){
	      for(a_j=0; a_j<agents->all_agents_no; a_j++) {
		  agent2 = agents->all_agents[a_j];

		  ///Do not compute for agent oneself.
		  if( (kn_on_ent->entPt->type == HRI_AGENT_PART) || (kn_on_ent->entPt->type == HRI_ISAGENT) ) {
		      if( agent2 == agents->all_agents[kn_on_ent->entPt->agent_idx] )
			  continue;
		  }

		  // Visibile - Invisible
		  switch(kn_on_ent->visibilityBy[a_j]) {
		  case HRI_VISIBLE:
		      printf("%s isVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  case HRI_INVISIBLE:
		      printf("%s isNotVisibleBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  case HRI_UK_VIS:
		      printf("%s isVisibleBy %s unknown", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  }

		  // ReachabilityBy - Commented out for performance reasons
		  // TODO: Make a difference in ORO for HARDLY_REACHABLE

		  switch(kn_on_ent->reachabilityBy[a_j]) {
		  case HRI_REACHABLE:
		      printf("%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  case HRI_HARDLY_REACHABLE:
		      printf("%s isReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  case HRI_UNREACHABLE:
		      printf("%s isNotReachableBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  case HRI_UK_REACHABILITY:
		      printf("%s isReachableBy %s unknown", kn_on_ent->entPt->name,agent2->robotPt->name);
		      break;
		  }
	
		  // LOOKSAT

		  if(kn_on_ent->is_looked_atBy[a_j] == HRI_TRUE_V) {
		      printf("%s isLookedAtBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		  }

		  // ISINFIELDOFVIEW
		  if(kn_on_ent->isSeenBy[a_j] == HRI_TRUE_V) {
		      printf("%s isSeenBy %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		  }

		  // POINTSAT
		  if(kn_on_ent->is_pointed_atBy[a_j] == HRI_TRUE_V) {
		      printf("%s isPointedAt %s", kn_on_ent->entPt->name,agent2->robotPt->name);
		  }

	      }
	  }


  
	  // FRONT - BACK - ETC
	  if(kn_on_ent->is_left_right_from_agent == HRI_LEFT) {
	    printf("%s leftOf %s\n", kn_on_ent->entPt->name , agent->robotPt->name);
	  }
	  else if(kn_on_ent->is_left_right_from_agent == HRI_RIGHT) {
	    printf("%s rightOf %s\n", kn_on_ent->entPt->name , agent->robotPt->name);
	  }
		    
	  if(kn_on_ent->is_front_behind_from_agent == HRI_FRONT) {
	    printf("%s inFrontOf %s\n", kn_on_ent->entPt->name , agent->robotPt->name);		      }
	  else if(kn_on_ent->is_front_behind_from_agent == HRI_BEHIND) {
	    printf("%s behind %s\n", kn_on_ent->entPt->name , agent->robotPt->name);
	  }
		    
	  if(kn_on_ent->is_far_near_from_agent == HRI_FAR) {
	    printf("%s cyc:farFrom %s\n", kn_on_ent->entPt->name , agent->robotPt->name);
	  }
	  else if(kn_on_ent->is_far_near_from_agent == HRI_NEAR) {
	    printf("%s cyc:near %s\n", kn_on_ent->entPt->name , agent->robotPt->name);
	  }

	  // IN - ON - NEXTTO
	  for(e_j=0; e_j<kn_on_ent->is_placed_nb; e_j++) {
	    // placement relation between the same object has no meaning
	    if (e_j != e_i) {
	      // delete previous value. ( right now only on placement relation is possible )
	      switch(kn_on_ent->is_placed[e_j]) {
	      case HRI_ISIN:
		printf("%s isIn %s\n", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		break;
	      case HRI_ISON:
		printf("%s isOn %s\n", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		break;
	      case HRI_ISNEXTTO:
		printf("%s isNextTo %s\n", kn_on_ent->entPt->name, ents->entities[e_j]->name);
		break;
	      case HRI_UK_PLR:
		//Nothing to do
		break;
	      case HRI_NOPLR:
		//Nothing to do?
		break;
	      }
	    }
	  }
	}
      }
    }
    else
	printf("Agent %s is not present. No Prints of Oro Facts for this Agent\n", agent->robotPt->name);
  }
}


#if defined(USE_MIGHTABILITY_MAPS)
/// Matthieu Warnier 20 April 2011
/// Function that decide how to update isMoving given Agent_Activity last and new state
SPARK_AGENT_ACTIVITY_MOTION_TRANSITION analyseAgentActivityMotionTransition( SPARK_AGENT_ACTIVITY_MOTION_TYPE oldOroValue ,  SPARK_AGENT_ACTIVITY_MOTION_TYPE newAgActivityValue )
{
  
  /// oldOroValue is SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN , SPARK_AGENT_ACTIVITY_MOTION_STATIC or SPARK_AGENT_ACTIVITY_MOTION_MOVING.
  /// (in Oro Value , new spark Value ) --> (Transition , new in Oro value)

  /// (SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN ,  ANY VALUE ) --
  /// --> (SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE , SPARK_AGENT_ACTIVITY_MOTION_STATIC) 

  /// (SPARK_AGENT_ACTIVITY_MOTION_STATIC ,  ANY VALUE except MOVING) --
  /// --> (SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE , SPARK_AGENT_ACTIVITY_MOTION_STATIC) 

  /// (SPARK_AGENT_ACTIVITY_MOTION_STATIC , SPARK_AGENT_ACTIVITY_MOTION_MOVING ) --
  /// --> (SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE , SPARK_AGENT_ACTIVITY_MOTION_MOVING) 

  /// (SPARK_AGENT_ACTIVITY_MOTION_MOVING ,  ANY VALUE except STATIC ) --
  /// --> (SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE , SPARK_AGENT_ACTIVITY_MOTION_MOVING) 

  /// (SPARK_AGENT_ACTIVITY_MOTION_MOVING , SPARK_AGENT_ACTIVITY_MOTION_STATIC ) --
  /// --> (SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE , SPARK_AGENT_ACTIVITY_MOTION_STATIC) 


  SPARK_AGENT_ACTIVITY_MOTION_TRANSITION agActivityMotionTransition;

  /// SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE conditions
  if ( (oldOroValue == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN))
    agActivityMotionTransition = SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE;
  else if ( (oldOroValue == SPARK_AGENT_ACTIVITY_MOTION_STATIC) && (newAgActivityValue !=  SPARK_AGENT_ACTIVITY_MOTION_MOVING))
    agActivityMotionTransition = SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE;
  else if ( (oldOroValue == SPARK_AGENT_ACTIVITY_MOTION_STATIC) && (newAgActivityValue ==  SPARK_AGENT_ACTIVITY_MOTION_MOVING))
    agActivityMotionTransition = SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE;
  else if ( (oldOroValue == SPARK_AGENT_ACTIVITY_MOTION_MOVING) && (newAgActivityValue !=  SPARK_AGENT_ACTIVITY_MOTION_STATIC))
    agActivityMotionTransition = SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE;
  else if ( (oldOroValue == SPARK_AGENT_ACTIVITY_MOTION_MOVING) && (newAgActivityValue ==  SPARK_AGENT_ACTIVITY_MOTION_STATIC))
    agActivityMotionTransition = SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE;

  return agActivityMotionTransition;
}

//  ACHILE_HUMAN1_EYES
// ACHILE_HUMAN1_LEFTHAND
// ACHILE_HUMAN1_RIGHTHAND
// ACHILE_HUMAN1_TORSO


//
// Function to force reexport to oro in case of oro crashed
//
void reinitAgentsActivityFactsForOROExportControl(HRI_AGENTS * agents,int agActivityFactAgentIndex)
{
  agActivityFactInOro[agActivityFactAgentIndex].whole_body = AGENT_MOTION_STATUS_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].torso = AGENT_TORSO_STATUS_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].head = AGENT_HEAD_STATUS_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].left_hand = AGENT_HAND_STATUS_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].right_hand = AGENT_HAND_STATUS_UNKNOWN;
  agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
}

/// Functions to prepare facts on one agent activity to be sent to oro
/// This facts are put in the perceiving robot model
int prepareAgentsActivityFactsForORO(HRI_AGENTS * agents, agents_activity_facts * agActivityFact ,int agActivityFactAgentIndex)
{
  int i;
  char tempBuffer[100];
  agent_stmt AS;
  HRI_AGENT * agent;
  SPARK_AGENT_ACTIVITY_MOTION_TRANSITION transitionResult;
  int agentId = agents->source_agent_idx;

  GEN_STRING64 agentWithActivityName;

  // Test that index are positives
  if( (agActivityFactAgentIndex >= 0) && (agActivityFact[agActivityFactAgentIndex].agent_index >= 0)){

    // Get Name of agent whose activity is considered
    strcpy(agentWithActivityName.name,gEnv->robot[agActivityFact[agActivityFactAgentIndex].agent_index]->name);

    //Get perceiving Agent
    agent = agents->all_agents[agentId];
  
    //check it is indeed an agent to avoid bug if AgentStateAnalysis is not running.
    for(i=0; i<agents->all_agents_no; i++){
      if( strcasestr(agents->all_agents[i]->robotPt->name,agentWithActivityName.name))
	break;
    }
    // If it is not an agent, we exit.
    if( i == agents->all_agents_no)
      return FALSE;

    // initialization of sets
    if ( agents_stmts.empty() ) {
      AS.agentID = string(agent->robotPt->name);
      agents_stmts.push_back(AS);
    }


    ///All Agents have to be put as uninitialized
    if(!agActivityFactInOroInitialized){
      for(i=0; i<MAXI_NUM_OF_AGENT_FOR_HRI_TASK; i++) {
	agActivityFactInOroAgentInitialized[i]=FALSE;
      }
      agActivityFactInOroInitialized = TRUE;
    }

    ///Agent specific initialization.
    /// initialized Agent Activity values in oro to unknown values.
    if(!agActivityFactInOroAgentInitialized[agActivityFactAgentIndex]){

      agActivityFactInOro[agActivityFactAgentIndex].agent_index = agActivityFact[agActivityFactAgentIndex].agent_index;

      agActivityFactInOro[agActivityFactAgentIndex].whole_body = AGENT_MOTION_STATUS_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].torso = AGENT_TORSO_STATUS_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].head = AGENT_HEAD_STATUS_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].left_hand = AGENT_HAND_STATUS_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].right_hand = AGENT_HAND_STATUS_UNKNOWN;
      agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
      agActivityFactInOroAgentInitialized[agActivityFactAgentIndex] = TRUE;
    }



    // If the agent is in the scene
    if(agent->is_present) {

      ///Check Agent Index is uptodate
      if( agActivityFactInOro[agActivityFactAgentIndex].agent_index != agActivityFact[agActivityFactAgentIndex].agent_index)
	agActivityFactInOro[agActivityFactAgentIndex].agent_index = agActivityFact[agActivityFactAgentIndex].agent_index;

      // printf(" Whole Body: %d \n",agActivityFact[agActivityFactAgentIndex].whole_body);
      // printf(" Torso: %d \n",agActivityFact[agActivityFactAgentIndex].torso);
      // printf(" Head: %d \n",agActivityFact[agActivityFactAgentIndex].head);

      // printf(" Right Hand: %d \n",agActivityFact[agActivityFactAgentIndex].right_hand);
      // printf(" Right Hand: %d \n",agActivityFact[agActivityFactAgentIndex].right_hand_mode);
      // printf(" Left Hnad: %d \n",agActivityFact[agActivityFactAgentIndex].left_hand);
      // printf(" Left Hnad: %d \n",agActivityFact[agActivityFactAgentIndex].left_hand_mode);

      // printf(" ========= Send Agent Activit Status for agent to ORO %s =======\n",agentWithActivityName.name);

      // Whole Body Motion.
      transitionResult = analyseAgentActivityMotionTransition( (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].whole_body, (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFact[agActivityFactAgentIndex].whole_body);

      if(transitionResult == SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE){
        sprintf(tempBuffer,"%s isMoving true", agentWithActivityName.name);
        agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
        agActivityFactInOro[agActivityFactAgentIndex].whole_body = AGENT_MOVING;
      }else if(transitionResult == SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE){
        sprintf(tempBuffer,"%s isMoving false", agentWithActivityName.name);
        agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
        agActivityFactInOro[agActivityFactAgentIndex].whole_body = AGENT_STATIC;
      }

      // Torso Motion.
      transitionResult = analyseAgentActivityMotionTransition( (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].torso, (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFact[agActivityFactAgentIndex].torso);
      if(transitionResult == SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE){
        sprintf(tempBuffer,"%s_TORSO isMoving true", agentWithActivityName.name);
        agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
        agActivityFactInOro[agActivityFactAgentIndex].torso = AGENT_TORSO_TURNING;
      }else if(transitionResult == SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE){
        sprintf(tempBuffer,"%s_TORSO isMoving false", agentWithActivityName.name);
        agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
        agActivityFactInOro[agActivityFactAgentIndex].torso = AGENT_TORSO_STATIC;
      }

      // Head Motion.
      transitionResult = analyseAgentActivityMotionTransition( (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].head, (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFact[agActivityFactAgentIndex].head);
 
 

      switch(transitionResult) {
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE:
	sprintf(tempBuffer,"%s_HEAD isMoving true", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].head = AGENT_HEAD_TURNING; 
	break;
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE:
	sprintf(tempBuffer,"%s_HEAD isMoving false", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].head = AGENT_HEAD_STATIC;
	break;
      }

      // Right Hand Motion.
      transitionResult = analyseAgentActivityMotionTransition( (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].right_hand, (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFact[agActivityFactAgentIndex].right_hand);

      switch(transitionResult) {
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE:
	sprintf(tempBuffer,"%s_RIGHTHAND isMoving true", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].right_hand = AGENT_HAND_MOVING;
	break;
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE:
	sprintf(tempBuffer,"%s_RIGHTHAND isMoving false", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].right_hand = AGENT_HAND_STATIC;   
	break;
      }

      // Left Hand Motion.
      transitionResult = analyseAgentActivityMotionTransition( (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].left_hand, (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFact[agActivityFactAgentIndex].left_hand);

      switch(transitionResult) {
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE:
	sprintf(tempBuffer,"%s_LEFTHAND isMoving true", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].left_hand = AGENT_HAND_MOVING;
	break;
      case SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE:
	sprintf(tempBuffer,"%s_LEFTHAND isMoving false", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToUpdate.insert(Statement(tempBuffer));
	agActivityFactInOro[agActivityFactAgentIndex].left_hand = AGENT_HAND_STATIC;
	break;
      }

      // ///Right hand mode
      // if( agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode != agActivityFact[agActivityFactAgentIndex].right_hand_mode ) {
      // 	switch(agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode) {
      // 	case AGENT_HAND_AT_REST_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes RIGHTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION rdf:type Rest", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION doneBy %s_RIGHTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    break;
      // 	case AGENT_HAND_AT_MANIPULATION_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes RIGHTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION rdf:type Manipulation");
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION doneBy %s_RIGHTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);


      // 	    break;
      // 	}
      // 	switch(agActivityFact[agActivityFactAgentIndex].right_hand_mode) {
      // 	case AGENT_HAND_AT_REST_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes RIGHTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION rdf:type Rest", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION doneBy %s_RIGHTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    break;
      // 	case AGENT_HAND_AT_MANIPULATION_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes RIGHTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION rdf:type Manipulation");
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"RIGHTHAND_ACTION doneBy %s_RIGHTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    break;
      // 	}
      // 	/// Update use for oro facts production value with current value
      // 	agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode = agActivityFact[agActivityFactAgentIndex].right_hand_mode;
      // }

      // ///Left hand mode
      // if( agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode != agActivityFact[agActivityFactAgentIndex].left_hand_mode ) {
      // 	switch(agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode) {
      // 	case AGENT_HAND_AT_REST_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes LEFTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION rdf:type Rest", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION doneBy %s_LEFTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    break;
      // 	case AGENT_HAND_AT_MANIPULATION_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes LEFTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION rdf:type Manipulation");
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION doneBy %s_LEFTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
      // 	    break;
      // 	}
      // 	switch(agActivityFact[agActivityFactAgentIndex].left_hand_mode) {
      // 	case AGENT_HAND_AT_REST_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes LEFTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION rdf:type Rest", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION doneBy %s_LEFTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    break;
      // 	case AGENT_HAND_AT_MANIPULATION_MODE:
      // 	    sprintf(tempBuffer,"%s currentlyBodilyDoes LEFTHAND_ACTION", agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION rdf:type Manipulation");
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    sprintf(tempBuffer,"LEFTHAND_ACTION doneBy %s_LEFTHAND",agentWithActivityName.name);
      // 	    agents_stmts[agentId].stmtsToAdd.insert(Statement(tempBuffer));
      // 	    break;
      // 	}
      // 	/// Update use for oro facts production value with current value
      // 	agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode = agActivityFact[agActivityFactAgentIndex].left_hand_mode;
      // }
    }
    else {
      /// Delete all agent activity facts.

      if(agActivityFactInOro[agActivityFactAgentIndex].whole_body != AGENT_MOTION_STATUS_UNKNOWN){
	sprintf(tempBuffer,"%s isMoving ?val",agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].whole_body = AGENT_MOTION_STATUS_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].torso != AGENT_TORSO_STATUS_UNKNOWN){
	sprintf(tempBuffer,"%s_TORSO isMoving ?val",agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].torso = AGENT_TORSO_STATUS_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].head != AGENT_HEAD_STATUS_UNKNOWN){
	sprintf(tempBuffer,"%s_HEAD isMoving ?val",agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].head = AGENT_HEAD_STATUS_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].left_hand != AGENT_HAND_STATUS_UNKNOWN){
	sprintf(tempBuffer,"%s_LEFTHAND isMoving ?val",agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].left_hand = AGENT_HAND_STATUS_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode != AGENT_HAND_CONFIG_MODE_UNKNOWN){
	sprintf(tempBuffer,"%s_LEFTHAND currentlyBodilyDoes LEFTHAND_ACTION", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	sprintf(tempBuffer,"LEFTHAND_ACTION rdf:type ?val", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].right_hand != AGENT_HAND_STATUS_UNKNOWN){
	sprintf(tempBuffer,"%s_RIGHTHAND isMoving ?val",agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].right_hand = AGENT_HAND_STATUS_UNKNOWN;
      }

      if(agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode != AGENT_HAND_CONFIG_MODE_UNKNOWN){
	sprintf(tempBuffer,"%s_RIGHTHAND currentlyBodilyDoes RIGHTHAND_ACTION", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	sprintf(tempBuffer,"RIGHTHAND_ACTION rdf:type ?val", agentWithActivityName.name);
	agents_stmts[agentId].stmtsToClear.insert(tempBuffer);
	agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode = AGENT_HAND_CONFIG_MODE_UNKNOWN;
      }


    }
  }
  else { 
    if(!agActivityFactInOroInitialized){
      //Use this boolean to avoid printing in loop
      //Reason for the wrong index should be investigated.
      agActivityFactInOroInitialized = TRUE;
      printf("There is a negative number among these two array index agActivityFactAgentIndex and agActivityFact[agActivityFactAgentIndex].agent_index in prepareAgentsActivityFactsForORO function %d %d\n", agActivityFactAgentIndex , agActivityFact[agActivityFactAgentIndex].agent_index);
    }
  }
  //FIXME, is that ok?
  return TRUE;
}

/// Functions to prepare facts on one agent activity to be sent to oro
/// This facts are put in the perceiving robot model
void printAgentsActivityFactsForORO(HRI_AGENTS * agents,int agActivityFactAgentIndex)
{
  // int a_i, e_i, e_j;
  HRI_AGENT * agent;
  SPARK_AGENT_ACTIVITY_MOTION_TRANSITION transitionResult;
  int agentId = agents->source_agent_idx;

  GEN_STRING64 agentWithActivityName;
  SPARK_AGENT_ACTIVITY_MOTION_TYPE sparkMotionType;

  if(agActivityFactInOroAgentInitialized[agActivityFactAgentIndex]){

    // Get Name of agent whose activity is considered
    strcpy(agentWithActivityName.name,gEnv->robot[agActivityFactInOro[agActivityFactAgentIndex].agent_index]->name);

    //Get perceiving Agent

    agent = agents->all_agents[agentId];

    // If the agent is in the scene
    if(agent->is_present) {

      printf("*********************************\n", agentWithActivityName.name);
      printf("Facts Sent To Oro for agent : %s \n", agentWithActivityName.name);
      printf("*********************************\n", agentWithActivityName.name);

      /// Update use for oro facts production value with current value
      sparkMotionType = (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].whole_body;
      if((sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) )
	printf("%s isMoving true\n", agentWithActivityName.name);
      else if( sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATIC)
	printf("%s isMoving false\n", agentWithActivityName.name);
      else if(  sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN)
	printf("%s isMoving unknown\n", agentWithActivityName.name);
      else
	printf("Unmanaged SPARK_AGENT_ACTIVITY_MOTION_TYPE %d for isMoving for agent %s\n", sparkMotionType,agentWithActivityName.name);


      sparkMotionType = (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].torso;
      if((sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) )
	printf("%s_TORSO isMoving true\n", agentWithActivityName.name);
      else if( sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATIC)
	printf("%s_TORSO isMoving false\n", agentWithActivityName.name);
      else if(  sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN)
	printf("%s torso isMoving unknown\n", agentWithActivityName.name);
      else
	printf("Unmanaged SPARK_AGENT_ACTIVITY_MOTION_TYPE %d for torso isMoving for agent %s\n", sparkMotionType,agentWithActivityName.name);

      sparkMotionType = (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].head;
      if((sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) )
	printf("%s_HEAD isMoving true\n", agentWithActivityName.name);
      else if( sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATIC)
	printf("%s_HEAD isMoving false\n", agentWithActivityName.name);
      else if(  sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN)
	printf("%s head isMoving unknown\n", agentWithActivityName.name);
      else
	printf("Unmanaged SPARK_AGENT_ACTIVITY_MOTION_TYPE %d for head isMoving for agent %s\n", sparkMotionType,agentWithActivityName.name);

      sparkMotionType = (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].right_hand;
      if((sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) )
	printf("%s_RIGHTHAND isMoving true\n", agentWithActivityName.name);
      else if( sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATIC)
	printf("%s_RIGHTHAND isMoving false\n", agentWithActivityName.name);
      else if(  sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN)
	printf("%s right hand isMoving unknown\n", agentWithActivityName.name);
      else
	printf("Unmanaged SPARK_AGENT_ACTIVITY_MOTION_TYPE %d for right hand isMoving for agent %s\n", sparkMotionType,agentWithActivityName.name);

      sparkMotionType = (SPARK_AGENT_ACTIVITY_MOTION_TYPE) agActivityFactInOro[agActivityFactAgentIndex].left_hand;
      if((sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) || (sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_MOVING) )
	printf("%s_LEFTHAND isMoving true\n", agentWithActivityName.name);
      else if( sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATIC)
	printf("%s_LEFTHAND isMoving false\n", agentWithActivityName.name);
      else if(  sparkMotionType == SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN)
	printf("%s left hand isMoving unknown\n", agentWithActivityName.name);
      else
	printf("Unmanaged SPARK_AGENT_ACTIVITY_MOTION_TYPE %d for left hand isMoving for agent %s\n", sparkMotionType,agentWithActivityName.name);

      ///Right hand mode
      switch(agActivityFactInOro[agActivityFactAgentIndex].right_hand_mode) {
      case AGENT_HAND_AT_REST_MODE:
            printf("%s_RIGHTHAND currentlyBodilyDoes RIGHTHAND_ACTION\n", agentWithActivityName.name);
            printf("RIGHTHAND_ACTION rdf:type Rest\n", agentWithActivityName.name);
            break;
      case AGENT_HAND_AT_MANIPULATION_MODE:
            printf("%s_RIGHTHAND currentlyBodilyDoes RIGHTHAND_ACTION\n", agentWithActivityName.name);
            printf("RIGHTHAND_ACTION rdf:type Manipulation\n");
            break;
      default:
            break;
      }

      ///Left hand mode
      switch(agActivityFactInOro[agActivityFactAgentIndex].left_hand_mode) {
      case AGENT_HAND_AT_REST_MODE:
	printf("%s_LEFTHAND currentlyBodilyDoes LEFTHAND_ACTION\n", agentWithActivityName.name);
	printf("LEFTHAND_ACTION rdf:type Rest\n", agentWithActivityName.name);
	break;
      case AGENT_HAND_AT_MANIPULATION_MODE:
	printf("%s_LEFTHAND currentlyBodilyDoes LEFTHAND_ACTION\n", agentWithActivityName.name);
	printf("LEFTHAND_ACTION rdf:type Manipulation\n");
	break;
      }
    }
    else {
      /// Delete all agent activity facts.
      printf("All facts unknown for this unpresent agent %s\n",agentWithActivityName.name);
    }
  }
}
#endif

#endif
