/*****************************************************************************
*
* Emrah Akin Sisbot
* Aug 10 
*
* File : sparkm3d.h
* Brief: This file exports the variables of Move3D to genom module
*
******************************************************************************/

#ifndef SPARKM3D_H
#define SPARKM3D_H

/*----------------------------------------------------------------------------
  INCLUDE
-----------------------------------------------------------------------------*/

#include <Util-pkg.h>
#include <P3d-pkg.h>
#include <Planner-pkg.h>
#include <Localpath-pkg.h>
#include <Collision-pkg.h>
#include <Graphic-pkg.h>
#include <LightPlanner-pkg.h>

#include <libmove3d/hri/hri.h>

#include <sys/time.h>

#ifdef HRI_COSTSPACE
#include<sstream>
extern std::string global_ActiveRobotName;
#include "API/Graphic/drawModule.hpp"
#include "HRI_costspace/HRICS_costspace.hpp"
#endif

#define D2D(x1,y1,x2,y2) (sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)))
#define D3D(x1,y1,z1,x2,y2,z2) (sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1)))

#define COND_PRINT(cond,str)  if( cond ) printf str
#define DEBUG_PRINTF(level,str)  if(SDI_F->State.debug_mode && (level <= SDI_F->State.debug_mode) ) printf str

extern hri_bitmapset * BTSET;
extern hri_bitmapset * ACBTSET; // ACBTSET is a global variable used for mightabilities

extern p3d_env * gEnv;
extern HRI_AGENT * gRobot;
extern hri_bitmapset * gBtset;
extern hri_gik * gGik;
extern HRI_AGENTS * gAgents;
extern HRI_ENTITIES * gEntities;
extern HRI_ACTION_MONITORING_SPHERES * gSpheres;
extern POSTER_ID gVimanPosterID;  

extern int Mightability_Maps_Initialized;
extern bool initSpheresIsDone;
extern std::string hri_text_to_display;

extern bool isKinectLocalizedByPom;

/*----------------------------------------------------------------------------
  STRUCTURES
  -----------------------------------------------------------------------------*/
 
/* typedef struct STRUCT_SPARK_ALL_REACHABILITIES { */
/*   SPARK_AGENT_REACHABILITY * agentReach; */
/*   int agentReachNb; */
/* } SPARK_ALL_REACHABILITIES;  */

typedef struct STRUCT_SPARK_AGENT_HISTORY_UNIT {
  SPARK_6D_COORD head;
  SPARK_3D_COORD lhand;
  SPARK_3D_COORD rhand;
  unsigned long headSensorTime;
} SPARK_AGENT_HISTORY_UNIT;

typedef struct STRUCT_SPARK_AGENT_HISTORY {
  SPARK_AGENT_HISTORY_UNIT history[SPARK_HUMAN_HISTORY_LENGTH];
  int length;
} SPARK_AGENT_HISTORY;

typedef struct STRUCT_SPARK_AGENT_SENSOR_DATA{
  int positionAv;
  int headAv;
  SPARK_6D_COORD head;
  int rightHandAv;
  SPARK_3D_COORD rightHand;
  int leftHandAv;
  SPARK_3D_COORD leftHand;
} SPARK_AGENT_SENSOR_DATA;


/*----------------------------------------------------------------------------
  PROTOTYPE
-----------------------------------------------------------------------------*/
extern int timeval_subtract(struct timeval *result, struct timeval *t2, struct timeval *t1);
extern void draw_spark_backbuffer( void );
extern void initialize_interface( void ); 
extern void publish_environment_poster( void );
extern void publish_position_update_management_poster( void );
extern void interface_MainLoopEvent();
extern bool getBBPoints( p3d_rob* rob,SPARK_BB_POINTS *bbPoints);


/// used by request time info
extern double getTimeInSecondMilisecond();
extern void getRequestTimeStart(int printLoopRequestExecutionTimeEnabled, double& timeStart);
extern void printRequestTime(int index);
extern void trackRequestTime(int printLoopRequestExecutionTimeEnabled, double timeStart, int& requestTimeId);

/// used by visualisation info 
extern int UpdateSomeVisuInfoForReachability( int agentIndex,int entityIndex , bool isReachable );
extern int UpdateSomeVisuInfoForDivergentBelief( int agentIndex,int entityIndex , bool hasDistinctLocation , bool hasKnownLocation );
extern void publishVisuInfoPoster( void );
extern void CheckVisuInfoForReachabilityPosition( void);

/// vector functions used by visualisaiton info
extern void createVec(double x,double y, double z,SPARK_3D_COORD & newVec);
extern void copyVec(SPARK_3D_COORD vec,SPARK_3D_COORD & newVec);
extern double getDistanceBetweenPoints(SPARK_3D_COORD point1,SPARK_3D_COORD point2);
/*----------------------------------------------------------------------------
  PROTOTYPE in sparkEnvironementCodels.cc
-----------------------------------------------------------------------------*/
extern int place_robot_conf_to_environment_poster(p3d_rob* rob, configPt q);
extern void sparkComputeSafetyCost();
extern int constructP3dToRobotNumIndexTable();

/*----------------------------------------------------------------------------
  PROTOTYPE in sparkSituationAssessmentCodels.cc
-----------------------------------------------------------------------------*/
extern bool getAgentPanTiltAndFourMostLookedAtPositionStates(double & pan,double & tilt, SPARK_DIST_ELEM_TYPE & positionStatePosition1Type, int & positionStatePosition1Index, double & distance1, SPARK_DIST_ELEM_TYPE & positionStatePosition2Type, int & positionStatePosition2Index, double & distance2, SPARK_DIST_ELEM_TYPE & positionStatePosition3Type, int & positionStatePosition3Index, double & distance3, SPARK_DIST_ELEM_TYPE & positionStatePosition4Type, int & positionStatePosition4Index, double & distance4, SPARK_DIST_ELEM_TYPE & containerType, int & containerObjectIndex, double & containerDistance);

extern void getObjectPositionFourClosestPositionStates(double x,double y,double z, SPARK_DIST_ELEM_TYPE & positionStatePosition1Type, int & positionStatePosition1Index, double & distance1, SPARK_DIST_ELEM_TYPE & positionStatePosition2Type, int & positionStatePosition2Index, double & distance2, SPARK_DIST_ELEM_TYPE & positionStatePosition3Type, int & positionStatePosition3Index, double & distance3, SPARK_DIST_ELEM_TYPE & positionStatePosition4Type, int & positionStatePosition4Index, double & distance4, SPARK_DIST_ELEM_TYPE & containerType, int & containerObjectIndex, double & containerDistance);
#endif
